"""Test the Abode device classes."""
import unittest

import requests_mock

import jaraco.abode
import jaraco.abode.helpers.constants as CONST

import tests.mock.login as LOGIN
import tests.mock.oauth_claims as OAUTH_CLAIMS
import tests.mock.logout as LOGOUT
import tests.mock.panel as PANEL
import tests.mock.devices as DEVICES
import tests.mock.devices.valve as VALVE
import pytest


USERNAME = 'foobar'
PASSWORD = 'deadbeef'


class TestValve(unittest.TestCase):
    """Test the AbodePy valve."""

    def setUp(self):
        """Set up Abode module."""
        self.abode = jaraco.abode.Abode(
            username=USERNAME, password=PASSWORD, disable_cache=True
        )

    def tearDown(self):
        """Clean up after test."""
        self.abode = None

    @requests_mock.mock()
    def tests_switch_device_properties(self, m):
        """Tests that switch devices properties work as expected."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(
            CONST.DEVICES_URL,
            text=VALVE.device(
                devid=VALVE.DEVICE_ID,
                status=CONST.STATUS_CLOSED,
                low_battery=False,
                no_response=False,
            ),
        )

        # Logout to reset everything
        self.abode.logout()

        # Get our power switch
        device = self.abode.get_device(VALVE.DEVICE_ID)

        # Test our device
        assert device is not None
        assert device.status == CONST.STATUS_CLOSED
        assert not device.battery_low
        assert not device.no_response
        assert not device.is_on
        assert not device.is_dimmable

        # Set up our direct device get url
        device_url = str.replace(CONST.DEVICE_URL, '$DEVID$', VALVE.DEVICE_ID)

        # Change device properties
        m.get(
            device_url,
            text=VALVE.device(
                devid=VALVE.DEVICE_ID,
                status=CONST.STATUS_OPEN,
                low_battery=True,
                no_response=True,
            ),
        )

        # Refesh device and test changes
        device.refresh()

        assert device.status == CONST.STATUS_OPEN
        assert device.battery_low
        assert device.no_response
        assert device.is_on

    @requests_mock.mock()
    def tests_switch_status_changes(self, m):
        """Tests that switch device changes work as expected."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(
            CONST.DEVICES_URL,
            text=VALVE.device(
                devid=VALVE.DEVICE_ID,
                status=CONST.STATUS_CLOSED,
                low_battery=False,
                no_response=False,
            ),
        )

        # Logout to reset everything
        self.abode.logout()

        # Get our power switch
        device = self.abode.get_device(VALVE.DEVICE_ID)

        # Test that we have our device
        assert device is not None
        assert device.status == CONST.STATUS_CLOSED
        assert not device.is_on

        # Set up control url response
        control_url = CONST.BASE_URL + VALVE.CONTROL_URL
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid=VALVE.DEVICE_ID, status=CONST.STATUS_OPEN_INT
            ),
        )

        # Change the mode to "on"
        assert device.switch_on()
        assert device.status == CONST.STATUS_OPEN
        assert device.is_on

        # Change response
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid=VALVE.DEVICE_ID, status=CONST.STATUS_CLOSED_INT
            ),
        )

        # Change the mode to "off"
        assert device.switch_off()
        assert device.status == CONST.STATUS_CLOSED
        assert not device.is_on

        # Test that an invalid status response throws exception
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid=VALVE.DEVICE_ID, status=CONST.STATUS_CLOSED_INT
            ),
        )

        with pytest.raises(jaraco.abode.AbodeException):
            device.switch_on()



"""
An Abode alarm Python library.
"""

import json
import logging
import os
import requests
from requests.exceptions import RequestException

from .automation import AbodeAutomation
from .devices.binary_sensor import AbodeBinarySensor
from .devices.camera import AbodeCamera
from .devices.cover import AbodeCover
from .devices.light import AbodeLight
from .devices.lock import AbodeLock
from .devices.switch import AbodeSwitch
from .devices.sensor import AbodeSensor
from .devices.valve import AbodeValve
from .event_controller import AbodeEventController
from .exceptions import AbodeAuthenticationException, AbodeException
from .devices import alarm as ALARM
from .helpers import constants as CONST
from .helpers import errors as ERROR
from . import utils as UTILS

_LOGGER = logging.getLogger(__name__)


class Abode:
    """Main Abode class."""

    def __init__(
        self,
        username=None,
        password=None,
        auto_login=False,
        get_devices=False,
        get_automations=False,
        cache_path=CONST.CACHE_PATH,
        disable_cache=False,
    ):
        """Init Abode object."""
        self._session = None
        self._token = None
        self._panel = None
        self._user = None
        self._cache_path = cache_path
        self._disable_cache = disable_cache

        self._event_controller = AbodeEventController(self, url=CONST.SOCKETIO_URL)

        self._default_alarm_mode = CONST.MODE_AWAY

        self._devices = None

        self._automations = None

        # Create a requests session to persist the cookies
        self._session = requests.session()

        # Create a new cache template
        self._cache = {
            CONST.ID: None,
            CONST.PASSWORD: None,
            CONST.UUID: UTILS.gen_uuid(),
            CONST.COOKIES: None,
        }

        # Load and merge an existing cache
        if not disable_cache:
            self._load_cache()

        # If the username and password were passed in, update
        # the cache and save
        if username:
            self._cache[CONST.ID] = username

        if password:
            self._cache[CONST.PASSWORD] = password

        self._save_cache()

        # Load persisted cookies (which contains the UUID and the session ID)
        # if available
        if CONST.COOKIES in self._cache and self._cache[CONST.COOKIES] is not None:
            self._session.cookies = self._cache[CONST.COOKIES]

        if (
            self._cache[CONST.ID] is not None
            and self._cache[CONST.PASSWORD] is not None
            and auto_login
        ):
            self.login()

        if get_devices:
            self.get_devices()

        if get_automations:
            self.get_automations()

    def login(self, username=None, password=None, mfa_code=None):  # noqa: C901
        """Explicit Abode login."""
        if username is not None:
            self._cache[CONST.ID] = username
        if password is not None:
            self._cache[CONST.PASSWORD] = password

        if self._cache[CONST.ID] is None or not isinstance(self._cache[CONST.ID], str):
            raise AbodeAuthenticationException(ERROR.USERNAME)

        if self._cache[CONST.PASSWORD] is None or not isinstance(
            self._cache[CONST.PASSWORD], str
        ):
            raise AbodeAuthenticationException(ERROR.PASSWORD)

        self._save_cache()

        self._token = None

        login_data = {
            CONST.ID: self._cache[CONST.ID],
            CONST.PASSWORD: self._cache[CONST.PASSWORD],
            CONST.UUID: self._cache[CONST.UUID],
        }

        if mfa_code is not None:
            login_data[CONST.MFA_CODE] = mfa_code
            login_data['remember_me'] = 1

        response = self._session.post(CONST.LOGIN_URL, json=login_data)

        if response.status_code != 200:
            raise AbodeAuthenticationException((response.status_code, response.text))

        response_object = json.loads(response.text)

        # Check for multi-factor authentication
        if 'mfa_type' in response_object:
            if response_object['mfa_type'] == "google_authenticator":
                raise AbodeAuthenticationException(ERROR.MFA_CODE_REQUIRED)

            raise AbodeAuthenticationException(ERROR.UNKNOWN_MFA_TYPE)

        # Persist cookies (which contains the UUID and the session ID) to disk
        if self._session.cookies.get_dict():
            self._cache[CONST.COOKIES] = self._session.cookies
            self._save_cache()

        oauth_response = self._session.get(CONST.OAUTH_TOKEN_URL)

        if oauth_response.status_code != 200:
            raise AbodeAuthenticationException(
                (oauth_response.status_code, oauth_response.text)
            )

        oauth_response_object = json.loads(oauth_response.text)

        _LOGGER.debug("Login Response: %s", response.text)

        self._token = response_object['token']
        self._panel = response_object['panel']
        self._user = response_object['user']
        self._oauth_token = oauth_response_object['access_token']

        _LOGGER.info("Login successful")

        return True

    def logout(self):
        """Explicit Abode logout."""
        if self._token:
            header_data = {'ABODE-API-KEY': self._token}

            self._session = requests.session()
            self._token = None
            self._panel = None
            self._user = None
            self._devices = None
            self._automations = None

            try:
                response = self._session.post(CONST.LOGOUT_URL, headers=header_data)
                response_object = json.loads(response.text)
            except OSError as exc:
                _LOGGER.warning("Caught exception during logout: %s", str(exc))
                return False

            if response.status_code != 200:
                raise AbodeAuthenticationException(
                    (response.status_code, response_object['message'])
                )

            _LOGGER.debug("Logout Response: %s", response.text)

            _LOGGER.info("Logout successful")

        return True

    def refresh(self):
        """Do a full refresh of all devices and automations."""
        self.get_devices(refresh=True)
        self.get_automations(refresh=True)

    def get_devices(self, refresh=False, generic_type=None):  # noqa: C901
        """Get all devices from Abode."""
        if refresh or self._devices is None:
            if self._devices is None:
                self._devices = {}

            _LOGGER.info("Updating all devices...")
            response = self.send_request("get", CONST.DEVICES_URL)
            response_object = json.loads(response.text)

            if response_object and not isinstance(response_object, (tuple, list)):
                response_object = [response_object]

            _LOGGER.debug("Get Devices Response: %s", response.text)

            for device_json in response_object:
                # Attempt to reuse an existing device
                device = self._devices.get(device_json['id'])

                # No existing device, create a new one
                if device:
                    device.update(device_json)
                else:
                    device = new_device(device_json, self)

                    if not device:
                        _LOGGER.debug("Skipping unknown device: %s", device_json)

                        continue

                    self._devices[device.device_id] = device

            # We will be treating the Abode panel itself as an armable device.
            panel_response = self.send_request("get", CONST.PANEL_URL)
            panel_json = json.loads(panel_response.text)

            self._panel.update(panel_json)

            _LOGGER.debug("Get Mode Panel Response: %s", response.text)

            alarm_device = self._devices.get(CONST.ALARM_DEVICE_ID + '1')

            if alarm_device:
                alarm_device.update(self._panel)
            else:
                alarm_device = ALARM.create_alarm(self._panel, self)
                self._devices[alarm_device.device_id] = alarm_device

        if generic_type:
            devices = []
            for device in self._devices.values():
                if (
                    device.generic_type is not None
                    and device.generic_type in generic_type
                ):
                    devices.append(device)
            return devices

        return list(self._devices.values())

    def get_device(self, device_id, refresh=False):
        """Get a single device."""
        if self._devices is None:
            self.get_devices()
            refresh = False

        device = self._devices.get(device_id)

        if device and refresh:
            device.refresh()

        return device

    def get_automations(self, refresh=False):
        """Get all automations."""
        if refresh or self._automations is None:
            if self._automations is None:
                # Set up the device libraries
                self._automations = {}

            _LOGGER.info("Updating all automations...")
            response = self.send_request("get", CONST.AUTOMATION_URL)
            response_object = json.loads(response.text)

            if response_object and not isinstance(response_object, (tuple, list)):
                response_object = [response_object]

            _LOGGER.debug("Get Automations Response: %s", response.text)

            for automation_json in response_object:
                # Attempt to reuse an existing automation object
                automation = self._automations.get(str(automation_json['id']))

                # No existing automation, create a new one
                if automation:
                    automation.update(automation_json)
                else:
                    automation = AbodeAutomation(self, automation_json)
                    self._automations[automation.automation_id] = automation

        return list(self._automations.values())

    def get_automation(self, automation_id, refresh=False):
        """Get a single automation."""
        if self._automations is None:
            self.get_automations()
            refresh = False

        automation = self._automations.get(str(automation_id))

        if automation and refresh:
            automation.refresh()

        return automation

    def get_alarm(self, area='1', refresh=False):
        """Shortcut method to get the alarm device."""
        if self._devices is None:
            self.get_devices()
            refresh = False

        return self.get_device(CONST.ALARM_DEVICE_ID + area, refresh)

    def set_default_mode(self, default_mode):
        """Set the default mode when alarms are turned 'on'."""
        if default_mode.lower() not in (CONST.MODE_AWAY, CONST.MODE_HOME):
            raise AbodeException(ERROR.INVALID_DEFAULT_ALARM_MODE)

        self._default_alarm_mode = default_mode.lower()

    def set_setting(self, setting, value, area='1', validate_value=True):
        """Set an abode system setting to a given value."""
        setting = setting.lower()

        if setting not in CONST.ALL_SETTINGS:
            raise AbodeException(ERROR.INVALID_SETTING, CONST.ALL_SETTINGS)

        if setting in CONST.PANEL_SETTINGS:
            url = CONST.SETTINGS_URL
            data = self._panel_settings(setting, value, validate_value)
        elif setting in CONST.AREA_SETTINGS:
            url = CONST.AREAS_URL
            data = self._area_settings(area, setting, value, validate_value)
        elif setting in CONST.SOUND_SETTINGS:
            url = CONST.SOUNDS_URL
            data = self._sound_settings(area, setting, value, validate_value)
        elif setting in CONST.SIREN_SETTINGS:
            url = CONST.SIREN_URL
            data = self._siren_settings(setting, value, validate_value)

        return self.send_request(method="put", url=url, data=data)

    @staticmethod
    def _panel_settings(setting, value, validate_value):
        """Will validate panel settings and values, returns data packet."""
        if validate_value:
            if (
                setting == CONST.SETTING_CAMERA_RESOLUTION
                and value not in CONST.SETTING_ALL_CAMERA_RES
            ):
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.SETTING_ALL_CAMERA_RES
                )
            if (
                setting
                in [CONST.SETTING_CAMERA_GRAYSCALE, CONST.SETTING_SILENCE_SOUNDS]
                and value not in CONST.SETTING_DISABLE_ENABLE
            ):
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.SETTING_DISABLE_ENABLE
                )

        return {setting: value}

    @staticmethod
    def _area_settings(area, setting, value, validate_value):
        """Will validate area settings and values, returns data packet."""
        if validate_value:
            # Exit delay has some specific limitations apparently
            if (
                setting == CONST.SETTING_EXIT_DELAY_AWAY
                and value not in CONST.VALID_SETTING_EXIT_AWAY
            ):
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.VALID_SETTING_EXIT_AWAY
                )
            if value not in CONST.ALL_SETTING_ENTRY_EXIT_DELAY:
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.ALL_SETTING_ENTRY_EXIT_DELAY
                )

        return {'area': area, setting: value}

    @staticmethod
    def _sound_settings(area, setting, value, validate_value):
        """Will validate sound settings and values, returns data packet."""
        if validate_value:
            if (
                setting in CONST.VALID_SOUND_SETTINGS
                and value not in CONST.ALL_SETTING_SOUND
            ):
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.ALL_SETTING_SOUND
                )
            if (
                setting == CONST.SETTING_ALARM_LENGTH
                and value not in CONST.ALL_SETTING_ALARM_LENGTH
            ):
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.ALL_SETTING_ALARM_LENGTH
                )
            if (
                setting == CONST.SETTING_FINAL_BEEPS
                and value not in CONST.ALL_SETTING_FINAL_BEEPS
            ):
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.ALL_SETTING_FINAL_BEEPS
                )

        return {'area': area, setting: value}

    @staticmethod
    def _siren_settings(setting, value, validate_value):
        """Will validate siren settings and values, returns data packet."""
        if validate_value:
            if value not in CONST.SETTING_DISABLE_ENABLE:
                raise AbodeException(
                    ERROR.INVALID_SETTING_VALUE, CONST.SETTING_DISABLE_ENABLE
                )

        return {'action': setting, 'option': value}

    def send_request(self, method, url, headers=None, data=None, is_retry=False):
        """Send requests to Abode."""
        if not self._token:
            self.login()

        if not headers:
            headers = {}

        headers['Authorization'] = 'Bearer ' + self._oauth_token
        headers['ABODE-API-KEY'] = self._token

        try:
            response = getattr(self._session, method)(url, headers=headers, json=data)

            if response and response.status_code < 400:
                return response
        except RequestException:
            _LOGGER.info("Abode connection reset...")

        if not is_retry:
            # Delete our current token and try again -- will force a login
            # attempt.
            self._token = None

            return self.send_request(method, url, headers, data, True)

        raise AbodeException((ERROR.REQUEST))

    @property
    def default_mode(self):
        """Get the default mode."""
        return self._default_alarm_mode

    @property
    def events(self):
        """Get the event controller."""
        return self._event_controller

    @property
    def uuid(self):
        """Get the UUID."""
        return self._cache[CONST.UUID]

    def _get_session(self):
        # Perform a generic update so we know we're logged in
        self.send_request("get", CONST.PANEL_URL)

        return self._session

    def _load_cache(self):
        """Load existing cache and merge for updating if required."""
        if not self._disable_cache and os.path.exists(self._cache_path):
            _LOGGER.debug("Cache found at: %s", self._cache_path)
            loaded_cache = UTILS.load_cache(self._cache_path)

            if loaded_cache:
                UTILS.update(self._cache, loaded_cache)
            else:
                _LOGGER.debug("Removing invalid cache file: %s", self._cache_path)
                os.remove(self._cache_path)

        self._save_cache()

    def _save_cache(self):
        """Trigger a cache save."""
        if not self._disable_cache:
            UTILS.save_cache(self._cache, self._cache_path)


def _new_sensor(device_json, abode):
    statuses = device_json.get(CONST.STATUSES_KEY, {})

    if any(key in statuses for key in CONST.SENSOR_KEYS):
        device_json['generic_type'] = CONST.TYPE_SENSOR
        return AbodeSensor(device_json, abode)

    version = device_json.get('version', '')

    # this.version.startsWith('MINIPIR') == true ? 'Occupancy Sensor'
    # : 'Motion Sensor';
    if version and version.lower().startswith('minipir'):
        device_json['generic_type'] = CONST.TYPE_OCCUPANCY
    else:
        device_json['generic_type'] = CONST.TYPE_MOTION

    return AbodeBinarySensor(device_json, abode)


def new_device(device_json, abode):
    """Create new device object for the given type."""
    type_tag = device_json.get('type_tag')

    if not type_tag:
        raise AbodeException((ERROR.UNABLE_TO_MAP_DEVICE))

    generic_type = CONST.get_generic_type(type_tag.lower())
    device_json['generic_type'] = generic_type

    if generic_type in [
        CONST.TYPE_CONNECTIVITY,
        CONST.TYPE_MOISTURE,
        CONST.TYPE_OPENING,
    ]:
        return AbodeBinarySensor(device_json, abode)

    if generic_type == CONST.TYPE_CAMERA:
        return AbodeCamera(device_json, abode)

    if generic_type == CONST.TYPE_COVER:
        return AbodeCover(device_json, abode)

    if generic_type == CONST.TYPE_LIGHT:
        return AbodeLight(device_json, abode)

    if generic_type == CONST.TYPE_LOCK:
        return AbodeLock(device_json, abode)

    if generic_type == CONST.TYPE_SWITCH:
        return AbodeSwitch(device_json, abode)

    if generic_type == CONST.TYPE_VALVE:
        return AbodeValve(device_json, abode)

    if generic_type == CONST.TYPE_UNKNOWN_SENSOR:
        return _new_sensor(device_json, abode)



"""Mock Abode Power Switch Sensor Device."""
import jaraco.abode.helpers.constants as CONST

DEVICE_ID = 'ZB:00000106'
DEVICE_UUID = 'abcd33455232fff31232'
CONTROL_URL = 'api/v1/control/light/' + DEVICE_ID
INTEGRATIONS_URL = CONST.INTEGRATIONS_URL + DEVICE_UUID


def color_temp_post_response_ok(devid, color_temp):
    """Return color temp change response json."""
    return (
        '''
      {
         "idForPanel": "'''
        + devid
        + '''",
         "colorTemperature": '''
        + str(int(color_temp))
        + '''
      }'''
    )


def color_post_response_ok(devid, hue, saturation):
    """Return color change response json."""
    return (
        '''
      {
         "idForPanel": "'''
        + devid
        + '''",
         "hue": '''
        + str(int(hue))
        + ''',
         "saturation": '''
        + str(int(saturation))
        + '''
      }'''
    )


def device(
    devid=DEVICE_ID,
    uuid=DEVICE_UUID,
    status=CONST.STATUS_OFF,
    level=0,
    saturation=57,
    hue=60,
    color_temp=6536,
    color_mode=CONST.COLOR_MODE_OFF,
    low_battery=False,
    no_response=False,
):
    """Hue  mock device."""
    return (
        '''
      {
         "id":"'''
        + devid
        + '''",
         "type_tag":"device_type.hue",
         "type":"RGB Dimmer",
         "name":"Overhead Light",
         "area":"1",
         "zone":"30",
         "sort_order":"",
         "is_window":"",
         "bypass":"0",
         "schar_24hr":"0",
         "sresp_24hr":"0",
         "sresp_mode_0":"0",
         "sresp_entry_0":"0",
         "sresp_exit_0":"0",
         "group_name":"Ungrouped",
         "group_id":"1",
         "default_group_id":"1",
         "sort_id":"10000",
         "sresp_mode_1":"0",
         "sresp_entry_1":"0",
         "sresp_exit_1":"0",
         "sresp_mode_2":"0",
         "sresp_entry_2":"0",
         "sresp_exit_2":"0",
         "sresp_mode_3":"0",
         "uuid":"'''
        + DEVICE_UUID
        + '''",
         "sresp_entry_3":"0",
         "sresp_exit_3":"0",
         "sresp_mode_4":"0",
         "sresp_entry_4":"0",
         "sresp_exit_4":"0",
         "version":"LST002",
         "origin":"abode",
         "has_subscription":null,
         "control_url":"'''
        + CONTROL_URL
        + '''",
         "deep_link":null,
         "status_color":"#5cb85c",
         "faults":{
            "low_battery":'''
        + str(int(low_battery))
        + ''',
            "tempered":0,
            "supervision":0,
            "out_of_order":0,
            "no_response":'''
        + str(int(no_response))
        + ''',
            "jammed":0,
            "zwave_fault":0
         },
         "status":"'''
        + status
        + '''",
         "statuses":{
            "saturation":'''
        + str(int(saturation))
        + ''',
            "hue":'''
        + str(int(hue))
        + ''',
            "level":"'''
        + str(int(level))
        + '''",
            "switch":"1",
            "color_temp":'''
        + str(int(color_temp))
        + ''',
            "color_mode":"'''
        + str(int(color_mode))
        + '''"
         },
         "status_ex":"",
         "actions":[
            {
               "label":"Switch off",
               "value":"a=1&z=30&sw=off;"
            },
            {
               "label":"Switch on",
               "value":"a=1&z=30&sw=on;"
            },
            {
               "label":"Toggle",
               "value":"a=1&z=30&sw=toggle;"
            },
            {
               "label":"0%",
               "value":"a=1&z=30&sw=0;"
            },
            {
               "label":"10%",
               "value":"a=1&z=30&sw=10;"
            },
            {
               "label":"20%",
               "value":"a=1&z=30&sw=20;"
            },
            {
               "label":"30%",
               "value":"a=1&z=30&sw=30;"
            },
            {
               "label":"40%",
               "value":"a=1&z=30&sw=40;"
            },
            {
               "label":"50%",
               "value":"a=1&z=30&sw=50;"
            },
            {
               "label":"60%",
               "value":"a=1&z=30&sw=60;"
            },
            {
               "label":"70%",
               "value":"a=1&z=30&sw=70;"
            },
            {
               "label":"80%",
               "value":"a=1&z=30&sw=80;"
            },
            {
               "label":"90%",
               "value":"a=1&z=30&sw=90;"
            },
            {
               "label":"100%",
               "value":"a=1&z=30&sw=99;"
            }
         ],
         "status_icons":[
         ],
         "statusEx":"37",
         "icon":"assets/icons/bulb-1.svg"
      }'''
    )



"""Mock Abode Power Switch Sensor Device."""
import jaraco.abode.helpers.constants as CONST

DEVICE_ID = 'RF:00000008'


def device(
    devid=DEVICE_ID, status=CONST.STATUS_OFF, low_battery=False, no_response=False
):
    """Water sensor mock device."""
    return (
        '''
        {
           "id":"'''
        + devid
        + '''",
           "type_tag":"device_type.water_sensor",
           "type":"Water Sensor",
           "name":"Downstairs Bathroo",
           "area":"1",
           "zone":"26",
           "sort_order":"0",
           "is_window":"0",
           "bypass":"0",
           "schar_24hr":"1",
           "sresp_mode_0":"0",
           "sresp_entry_0":"0",
           "sresp_exit_0":"0",
           "sresp_mode_1":"0",
           "sresp_entry_1":"0",
           "sresp_exit_1":"0",
           "sresp_mode_2":"0",
           "sresp_entry_2":"0",
           "sresp_exit_2":"0",
           "sresp_mode_3":"0",
           "sresp_entry_3":"0",
           "sresp_exit_3":"0",
           "capture_mode":null,
           "origin":"abode",
           "control_url":"",
           "deep_link":null,
           "status_color":"#5cb85c",
           "faults":{
              "low_battery":'''
        + str(int(low_battery))
        + ''',
              "tempered":0,
              "supervision":0,
              "out_of_order":0,
              "no_response":'''
        + str(int(no_response))
        + '''
           },
           "status":"'''
        + status
        + '''",
           "statuses":{
              "hvac_mode":null
           },
           "status_ex":"",
           "actions":[
           ],
           "status_icons":[
           ],
           "icon":"assets/icons/water-value-shutoff.svg"
        }'''
    )



"""Abode cover device."""

from ..devices.switch import AbodeSwitch
from ..helpers import constants as CONST


class AbodeCover(AbodeSwitch):
    """Class to add cover functionality."""

    def switch_on(self):
        """Turn the switch on."""
        success = self.set_status(CONST.STATUS_OPEN_INT)

        if success:
            self._json_state['status'] = CONST.STATUS_OPEN

        return success

    def switch_off(self):
        """Turn the switch off."""
        success = self.set_status(CONST.STATUS_CLOSED_INT)

        if success:
            self._json_state['status'] = CONST.STATUS_CLOSED

        return success

    def open_cover(self):
        """Open the cover."""
        return self.switch_on()

    def close_cover(self):
        """Close the cover."""
        return self.switch_off()

    @property
    def is_open(self):
        """Get if the cover is open."""
        return self.is_on

    @property
    def is_on(self):
        """
        Get cover state.

        Assume cover is open.
        """
        return self.status not in CONST.STATUS_CLOSED



"""Mock Abode Power Switch Sensor Device."""
import jaraco.abode.helpers.constants as CONST

DEVICE_ID = 'ZW:00000113'
CONTROL_URL = 'api/v1/control/power_switch/' + DEVICE_ID


def device(
    devid=DEVICE_ID, status=CONST.STATUS_OPEN, low_battery=False, no_response=False
):
    """Valve mock device."""
    return (
        '''
      {
         "actions":[
            {
               "label":"Close",
               "value":"a=1&z=18&sw=off;"
            },
            {
               "label":"Open",
               "value":"a=1&z=18&sw=on;"
            }
         ],
         "area":"1",
         "bypass":"0",
         "control_url":"'''
        + CONTROL_URL
        + '''",
         "deep_link":null,
         "default_group_id":"1",
         "faults":{
            "low_battery":'''
        + str(int(low_battery))
        + ''',
            "tempered":0,
            "supervision":0,
            "out_of_order":0,
            "no_response":'''
        + str(int(no_response))
        + '''
         },
         "generic_type":"valve",
         "group_id":"xxxxx",
         "group_name":"Water leak",
         "has_subscription":null,
         "icon":"assets/icons/water-value-shutoff.svg",
         "id":"'''
        + devid
        + '''",
         "is_window":"",
         "name":"Water shut-off valve",
         "onboard":"0",
         "origin":"abode",
         "s2_dsk":"",
         "s2_grnt_keys":"",
         "s2_keys_valid":"",
         "s2_propty":"",
         "schar_24hr":"0",
         "sort_id":"6",
         "sort_order":"",
         "sresp_24hr":"0",
         "sresp_entry_0":"0",
         "sresp_entry_1":"0",
         "sresp_entry_2":"0",
         "sresp_entry_3":"0",
         "sresp_entry_4":"0",
         "sresp_exit_0":"0",
         "sresp_exit_1":"0",
         "sresp_exit_2":"0",
         "sresp_exit_3":"0",
         "sresp_exit_4":"0",
         "sresp_mode_0":"0",
         "sresp_mode_1":"0",
         "sresp_mode_2":"0",
         "sresp_mode_3":"0",
         "sresp_mode_4":"0",
         "status":"'''
        + status
        + '''",
         "status_color":"#5cb85c",
         "status_display":"Open",
         "status_ex":"",
         "status_icons":[

         ],
         "statuses":{
            "switch":"1"
         },
         "type":"Shutoff Valve",
         "type_tag":"device_type.valve",
         "uuid":"xxxxxxxxxxxxxxxxxxxxxxxxx",
         "version":"021f00030002",
         "zone":"18",
         "zwave_secure_protocol":""
      }'''
    )



"""
Test Abode system setup, shutdown, and general functionality.

Tests the system initialization and attributes of the main Abode system.
"""
import os
import json

import pytest
import requests

import jaraco.abode
import jaraco.abode.helpers.constants as CONST

import tests.mock as MOCK
import tests.mock.login as LOGIN
import tests.mock.oauth_claims as OAUTH_CLAIMS
import tests.mock.logout as LOGOUT
import tests.mock.panel as PANEL
import tests.mock.devices as DEVICES
import tests.mock.devices.door_contact as DOOR_CONTACT
import tests.mock.user as USER

USERNAME = 'foobar'
PASSWORD = 'deadbeef'


@pytest.fixture
def cache_path(tmp_path, request):
    request.instance.cache_path = tmp_path / 'cache.pickle'


@pytest.fixture(autouse=True)
def abode_objects(request):
    self = request.instance
    self.abode_no_cred = jaraco.abode.Abode(disable_cache=True)
    self.abode = jaraco.abode.Abode(
        username=USERNAME, password=PASSWORD, disable_cache=True
    )


@pytest.fixture
def m(requests_mock):
    return requests_mock


class TestAbode:
    """Test the Abode class."""

    def tests_initialization(self):
        """Verify we can initialize abode."""
        # pylint: disable=protected-access
        assert self.abode._cache[CONST.ID] == USERNAME
        # pylint: disable=protected-access
        assert self.abode._cache[CONST.PASSWORD] == PASSWORD

    def tests_no_credentials(self):
        """Check that we throw an exception when no username/password."""
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.login()

        # pylint: disable=protected-access
        self.abode_no_cred._cache[CONST.ID] = USERNAME
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.login()

    def tests_manual_login(self, m):
        """Check that we can manually use the login() function."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())

        self.abode_no_cred.login(username=USERNAME, password=PASSWORD)

        # pylint: disable=protected-access
        assert self.abode_no_cred._cache[CONST.ID] == USERNAME
        # pylint: disable=protected-access
        assert self.abode_no_cred._cache[CONST.PASSWORD] == PASSWORD

    def tests_manual_login_with_mfa(self, m):
        """Check that we can login with MFA code."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())

        self.abode_no_cred.login(username=USERNAME, password=PASSWORD, mfa_code=654321)

        # pylint: disable=protected-access
        assert self.abode_no_cred._cache[CONST.ID] == USERNAME
        # pylint: disable=protected-access
        assert self.abode_no_cred._cache[CONST.PASSWORD] == PASSWORD

    def tests_auto_login(self, m):
        """Test that automatic login works."""
        auth_token = MOCK.AUTH_TOKEN
        user_json = USER.get_response_ok()
        login_json = LOGIN.post_response_ok(auth_token, user_json)
        panel_json = PANEL.get_response_ok()

        m.post(CONST.LOGIN_URL, text=login_json)
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.get(CONST.PANEL_URL, text=panel_json)
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())

        abode = jaraco.abode.Abode(
            username='fizz',
            password='buzz',
            auto_login=True,
            get_devices=False,
            disable_cache=True,
        )

        # pylint: disable=W0212
        assert abode._cache[CONST.ID] == 'fizz'
        assert abode._cache[CONST.PASSWORD] == 'buzz'
        assert abode._token == MOCK.AUTH_TOKEN
        assert abode._panel == json.loads(panel_json)
        assert abode._user == json.loads(user_json)
        assert abode._devices is None
        assert abode._automations is None

        abode.logout()

        abode = None

    def tests_auto_fetch(self, m):
        """Test that automatic device and automation retrieval works."""
        auth_token = MOCK.AUTH_TOKEN
        user_json = USER.get_response_ok()
        login_json = LOGIN.post_response_ok(auth_token, user_json)
        panel_json = PANEL.get_response_ok()

        m.post(CONST.LOGIN_URL, text=login_json)
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.get(CONST.PANEL_URL, text=panel_json)
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.get(CONST.AUTOMATION_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())

        abode = jaraco.abode.Abode(
            username='fizz',
            password='buzz',
            auto_login=False,
            get_devices=True,
            get_automations=True,
            disable_cache=True,
        )

        # pylint: disable=W0212
        assert abode._cache[CONST.ID] == 'fizz'
        assert abode._cache[CONST.PASSWORD] == 'buzz'
        assert abode._token == MOCK.AUTH_TOKEN
        assert abode._user == json.loads(user_json)
        assert abode._panel is not None

        # Contains one device, our alarm
        assert abode._devices == {'area_1': abode.get_alarm()}

        # Contains no automations
        assert abode._automations == {}

        abode.logout()

        abode = None

    def tests_login_failure(self, m):
        """Test login failed."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_bad_request(), status_code=400)
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())

        # Check that we raise an Exception with a failed login request.
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.login(username=USERNAME, password=PASSWORD)

    def tests_login_mfa_required(self, m):
        """Tests login with MFA code required but not supplied."""
        m.post(
            CONST.LOGIN_URL,
            text=LOGIN.post_response_mfa_code_required(),
            status_code=200,
        )

        # Check that we raise an Exception when the MFA code is required
        # but not supplied
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.login(username=USERNAME, password=PASSWORD)

    def tests_login_bad_mfa_code(self, m):
        """Tests login with bad MFA code."""
        m.post(
            CONST.LOGIN_URL, text=LOGIN.post_response_bad_mfa_code(), status_code=400
        )

        # Check that we raise an Exception with a bad MFA code
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.login(
                username=USERNAME, password=PASSWORD, mfa_code=123456
            )

    def tests_login_unknown_mfa_type(self, m):
        """Tests login with unknown MFA type."""
        m.post(
            CONST.LOGIN_URL,
            text=LOGIN.post_response_unknown_mfa_type(),
            status_code=200,
        )

        # Check that we raise an Exception with an unknown MFA type
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.login(username=USERNAME, password=PASSWORD)

    def tests_logout_failure(self, m):
        """Test logout failed."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.post(
            CONST.LOGOUT_URL, text=LOGOUT.post_response_bad_request(), status_code=400
        )

        self.abode_no_cred.login(username=USERNAME, password=PASSWORD)

        # Check that we raise an Exception with a failed logout request.
        with pytest.raises(jaraco.abode.AbodeAuthenticationException):
            self.abode_no_cred.logout()

    def tests_logout_exception(self, m):
        """Test logout exception."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.post(CONST.LOGOUT_URL, exc=requests.exceptions.ConnectTimeout)

        self.abode.login()

        # Check that we eat the exception gracefully
        assert not self.abode.logout()

    def tests_full_setup(self, m):
        """Check that Abode is set up properly."""
        auth_token = MOCK.AUTH_TOKEN
        user_json = USER.get_response_ok()
        login_json = LOGIN.post_response_ok(auth_token, user_json)
        panel_json = PANEL.get_response_ok()

        m.post(CONST.LOGIN_URL, text=login_json)
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.get(CONST.PANEL_URL, text=panel_json)
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())

        self.abode.get_devices()

        # pylint: disable=protected-access
        original_session = self.abode._session

        # pylint: disable=W0212
        assert self.abode._cache[CONST.ID] == USERNAME
        assert self.abode._cache[CONST.PASSWORD] == PASSWORD
        assert self.abode._token == auth_token
        assert self.abode._user == json.loads(user_json)
        assert self.abode._panel is not None
        assert self.abode.get_alarm() is not None
        assert self.abode._get_session() is not None
        assert self.abode._get_session() == original_session
        assert self.abode.events is not None

        self.abode.logout()

        # pylint: disable=W0212
        assert self.abode._token is None
        assert self.abode._panel is None
        assert self.abode._user is None
        assert self.abode._devices is None
        assert self.abode._automations is None
        assert self.abode._session is not None
        assert self.abode._get_session() != original_session

    def tests_reauthorize(self, m):
        """Check that Abode can reauthorize after token timeout."""
        new_token = "FOOBAR"
        m.post(
            CONST.LOGIN_URL,
            [
                {
                    'text': LOGIN.post_response_ok(auth_token=new_token),
                    'status_code': 200,
                }
            ],
        )

        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())

        m.get(
            CONST.DEVICES_URL,
            [
                {'text': MOCK.response_forbidden(), 'status_code': 403},
                {'text': DEVICES.EMPTY_DEVICE_RESPONSE, 'status_code': 200},
            ],
        )
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Forces a device update
        self.abode.get_devices()

        # pylint: disable=W0212
        assert self.abode._token == new_token

    def tests_send_request_exception(self, m):
        """Check that send_request recovers from an exception."""
        new_token = "DEADBEEF"
        m.post(
            CONST.LOGIN_URL,
            [
                {
                    'text': LOGIN.post_response_ok(auth_token=new_token),
                    'status_code': 200,
                }
            ],
        )

        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())

        m.get(
            CONST.DEVICES_URL,
            [
                {'exc': requests.exceptions.ConnectTimeout},
                {'text': DEVICES.EMPTY_DEVICE_RESPONSE, 'status_code': 200},
            ],
        )
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Forces a device update
        self.abode.get_devices()

        # pylint: disable=W0212
        assert self.abode._token == new_token

    def tests_continuous_bad_auth(self, m):
        """Check that Abode won't get stuck with repeated failed retries."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.get(CONST.DEVICES_URL, text=MOCK.response_forbidden(), status_code=403)

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.get_devices()

    def tests_default_mode(self):
        """Test that the default mode fails if not of type home or away."""
        self.abode.set_default_mode(CONST.MODE_HOME)
        assert self.abode.default_mode == CONST.MODE_HOME

        self.abode.set_default_mode(CONST.MODE_AWAY)
        assert self.abode.default_mode == CONST.MODE_AWAY

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_default_mode('foobar')

    def test_all_device_refresh(self, m):
        """Check that device refresh works and reuses the same objects."""
        dc1_devid = 'RF:01'
        dc1a = DOOR_CONTACT.device(devid=dc1_devid, status=CONST.STATUS_ON)

        dc2_devid = 'RF:02'
        dc2a = DOOR_CONTACT.device(devid=dc2_devid, status=CONST.STATUS_OFF)

        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.DEVICES_URL, text='[' + dc1a + ',' + dc2a + ']')
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Reset
        self.abode.logout()

        # Get all devices
        self.abode.get_devices()

        # Get and check devices
        # pylint: disable=W0212
        dc1a_dev = self.abode.get_device(dc1_devid)
        assert json.loads(dc1a)['id'] == dc1a_dev.device_id

        dc2a_dev = self.abode.get_device(dc2_devid)
        assert json.loads(dc2a)['id'] == dc2a_dev.device_id

        # Change device states
        dc1b = DOOR_CONTACT.device(devid=dc1_devid, status=CONST.STATUS_OFF)

        dc2b = DOOR_CONTACT.device(devid=dc2_devid, status=CONST.STATUS_ON)

        m.get(CONST.DEVICES_URL, text='[' + dc1b + ',' + dc2b + ']')

        # Refresh all devices
        self.abode.get_devices(refresh=True)

        # Get and check devices again, ensuring they are the same object
        # Future note: "if a is b" tests that the object is the same
        # Thus asserting dc1a_dev is dc1b_dev tests if they are the same object
        dc1b_dev = self.abode.get_device(dc1_devid)
        assert json.loads(dc1b)['id'] == dc1b_dev.device_id
        assert dc1a_dev is dc1b_dev

        dc2b_dev = self.abode.get_device(dc2_devid)
        assert json.loads(dc2b)['id'] == dc2b_dev.device_id
        assert dc2a_dev is dc2b_dev

    def tests_settings_validation(self, m):
        """Check that device panel general settings are working."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.get(CONST.SETTINGS_URL, text=MOCK.generic_response_ok())

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting("fliptrix", "foobar")

    def tests_general_settings(self, m):
        """Check that device panel general settings are working."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.put(CONST.SETTINGS_URL, text=MOCK.generic_response_ok())

        try:
            self.abode.set_setting(
                CONST.SETTING_CAMERA_RESOLUTION, CONST.SETTING_CAMERA_RES_640_480
            )

            self.abode.set_setting(CONST.SETTING_CAMERA_GRAYSCALE, CONST.SETTING_ENABLE)

            self.abode.set_setting(CONST.SETTING_SILENCE_SOUNDS, CONST.SETTING_ENABLE)
        except jaraco.abode.AbodeException:
            self.fail("set_setting() raised AbodeException unexpectedly")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_CAMERA_RESOLUTION, "foobar")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_CAMERA_GRAYSCALE, "foobar")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_SILENCE_SOUNDS, "foobar")

    def tests_area_settings(self, m):
        """Check that device panel areas settings are working."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.put(CONST.AREAS_URL, text=MOCK.generic_response_ok())

        try:
            self.abode.set_setting(
                CONST.SETTING_ENTRY_DELAY_AWAY, CONST.SETTING_ENTRY_EXIT_DELAY_10SEC
            )

            self.abode.set_setting(
                CONST.SETTING_EXIT_DELAY_AWAY, CONST.SETTING_ENTRY_EXIT_DELAY_30SEC
            )

        except jaraco.abode.AbodeException:
            self.fail("set_setting() raised AbodeException unexpectedly")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_ENTRY_DELAY_AWAY, "foobar")

        # 10 seconds is invalid here
        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(
                CONST.SETTING_EXIT_DELAY_AWAY, CONST.SETTING_ENTRY_EXIT_DELAY_10SEC
            )

    def tests_sound_settings(self, m):
        """Check that device panel sound settings are working."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.put(CONST.SOUNDS_URL, text=MOCK.generic_response_ok())

        try:
            self.abode.set_setting(CONST.SETTING_DOOR_CHIME, CONST.SETTING_SOUND_LOW)

            self.abode.set_setting(
                CONST.SETTING_ALARM_LENGTH, CONST.SETTING_ALARM_LENGTH_2MIN
            )

            self.abode.set_setting(
                CONST.SETTING_FINAL_BEEPS, CONST.SETTING_FINAL_BEEPS_3SEC
            )

        except jaraco.abode.AbodeException:
            self.fail("set_setting() raised AbodeException unexpectedly")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_DOOR_CHIME, "foobar")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_ALARM_LENGTH, "foobar")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_FINAL_BEEPS, "foobar")

    def tests_siren_settings(self, m):
        """Check that device panel siren settings are working."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())
        m.put(CONST.SIREN_URL, text=MOCK.generic_response_ok())

        try:
            self.abode.set_setting(
                CONST.SETTING_SIREN_ENTRY_EXIT_SOUNDS, CONST.SETTING_ENABLE
            )

            self.abode.set_setting(
                CONST.SETTING_SIREN_CONFIRM_SOUNDS, CONST.SETTING_ENABLE
            )

            self.abode.set_setting(
                CONST.SETTING_SIREN_TAMPER_SOUNDS, CONST.SETTING_ENABLE
            )

        except jaraco.abode.AbodeException:
            self.fail("set_setting() raised AbodeException unexpectedly")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_SIREN_ENTRY_EXIT_SOUNDS, "foobar")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_SIREN_CONFIRM_SOUNDS, "foobar")

        with pytest.raises(jaraco.abode.AbodeException):
            self.abode.set_setting(CONST.SETTING_SIREN_TAMPER_SOUNDS, "foobar")

    @pytest.mark.usefixtures('cache_path')
    def tests_cookies(self, m):
        """Check that cookies are saved and loaded successfully."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Create abode
        abode = jaraco.abode.Abode(
            username='fizz',
            password='buzz',
            auto_login=False,
            get_devices=False,
            disable_cache=False,
            cache_path=self.cache_path,
        )

        # Mock cookie created by Abode after login
        cookie = requests.cookies.create_cookie(name='SESSION', value='COOKIE')
        # pylint: disable=protected-access
        abode._session.cookies.set_cookie(cookie)

        abode.login()

        # Test that our cookies are fully realized prior to login
        # pylint: disable=W0212
        assert abode._cache['id'] is not None
        assert abode._cache['password'] is not None
        assert abode._cache['uuid'] is not None
        assert abode._cache['cookies'] is not None

        # Test that we now have a cookies file
        assert os.path.exists(self.cache_path)

        # Copy our current cookies file and data
        first_cookies_data = abode._cache

        # New abode instance reads in old data
        abode = jaraco.abode.Abode(
            username='fizz',
            password='buzz',
            auto_login=False,
            get_devices=False,
            disable_cache=False,
            cache_path=self.cache_path,
        )

        # Test that the cookie data is the same
        assert abode._cache['uuid'] == first_cookies_data['uuid']

    @pytest.mark.usefixtures('cache_path')
    def test_empty_cookies(self, m):
        """Check that empty cookies file is loaded successfully."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Create an empty file
        self.cache_path.write_text('')

        # Cookies are created
        empty_abode = jaraco.abode.Abode(
            username='fizz',
            password='buzz',
            auto_login=True,
            get_devices=False,
            disable_cache=False,
            cache_path=self.cache_path,
        )

        # Test that some cache exists
        # pylint: disable=W0212
        assert empty_abode._cache['id'] is not None
        assert empty_abode._cache['password'] is not None
        assert empty_abode._cache['uuid'] is not None

    @pytest.mark.usefixtures('cache_path')
    def test_invalid_cookies(self, m):
        """Check that empty cookies file is loaded successfully."""
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.DEVICES_URL, text=DEVICES.EMPTY_DEVICE_RESPONSE)
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Create an invalid pickle file
        self.cache_path.write_text('Invalid file goes here')

        # Cookies are created
        empty_abode = jaraco.abode.Abode(
            username='fizz',
            password='buzz',
            auto_login=True,
            get_devices=False,
            disable_cache=False,
            cache_path=self.cache_path,
        )

        # Test that some cache exists
        # pylint: disable=W0212
        assert empty_abode._cache['id'] is not None
        assert empty_abode._cache['password'] is not None
        assert empty_abode._cache['uuid'] is not None



"""Mock Abode User Response."""


def get_response_ok():
    """Return the user response data."""
    return '''{
        "id":"user@email.com",
        "email":"user@email.com",
        "first_name":"John",
        "last_name":"Doe",
        "phone":"5555551212",
        "profile_pic":"https://website.com/default-image.svg",
        "address":"555 None St.",
        "city":"New York City",
        "state":"NY",
        "zip":"10108",
        "country":"US",
        "longitude":"0",
        "latitude":"0",
        "timezone":"America/New_York_City",
        "verified":"1",
        "plan":"Basic",
        "plan_id":"0",
        "plan_active":"1",
        "cms_code":"1111",
        "cms_active":"0",
        "cms_started_at":"",
        "cms_expiry":"",
        "cms_ondemand":"",
        "step":"-1",
        "cms_permit_no":"",
        "opted_plan_id":"",
        "stripe_account":"1",
        "plan_effective_from":"",
        "agreement":"1",
        "associate_users":"1",
        "owner":"1"
        }'''



"""Command-line interface."""

import json
import logging
import time
import argparse
import contextlib

from . import Abode
from .helpers import constants as CONST
from .helpers import timeline as TIMELINE
from .exceptions import AbodeException

_LOGGER = logging.getLogger('abodecl')


def setup_logging(log_level=logging.INFO):
    """Set up the logging."""
    logging.basicConfig(level=log_level)
    fmt = "%(asctime)s %(levelname)s (%(threadName)s) " "[%(name)s] %(message)s"
    colorfmt = "%(log_color)s{}%(reset)s".format(fmt)
    datefmt = '%Y-%m-%d %H:%M:%S'

    # Suppress overly verbose logs from libraries that aren't helpful
    logging.getLogger('requests').setLevel(logging.WARNING)
    logging.getLogger('urllib3').setLevel(logging.WARNING)
    logging.getLogger('aiohttp.access').setLevel(logging.WARNING)

    try:
        from colorlog import ColoredFormatter

        logging.getLogger().handlers[0].setFormatter(
            ColoredFormatter(
                colorfmt,
                datefmt=datefmt,
                reset=True,
                log_colors={
                    'DEBUG': 'cyan',
                    'INFO': 'green',
                    'WARNING': 'yellow',
                    'ERROR': 'red',
                    'CRITICAL': 'red',
                },
            )
        )
    except ImportError:
        pass

    logger = logging.getLogger('')
    logger.setLevel(log_level)


def build_parser():
    """
    Get parsed arguments.

    >>> parser = build_parser()
    """
    parser = argparse.ArgumentParser("jaraco.abode: Command Line Utility")

    parser.add_argument('-u', '--username', help='Username')

    parser.add_argument('-p', '--password', help='Password')

    parser.add_argument('--mfa', help='Multifactor authentication code')

    parser.add_argument(
        '--cache',
        metavar='pickle_file',
        help='Create/update/use a pickle cache for the username and password.',
        default=CONST.CACHE_PATH,
    )

    parser.add_argument(
        '--mode',
        help='Output current alarm mode',
        default=False,
        action="store_true",
    )

    parser.add_argument(
        '--arm',
        metavar='mode',
        help='Arm alarm to mode',
    )

    parser.add_argument(
        '--set',
        metavar='setting=value',
        help='Set setting to a value',
        action='append',
    )

    parser.add_argument(
        '--devices',
        help='Output all devices',
        default=False,
        action="store_true",
    )

    parser.add_argument(
        '--device',
        metavar='device_id',
        help='Output one device for device_id',
        action='append',
    )

    parser.add_argument(
        '--json',
        metavar='device_id',
        help='Output the json for device_id',
        action='append',
    )

    parser.add_argument(
        '--on',
        metavar='device_id',
        help='Switch on a given device_id',
        action='append',
    )

    parser.add_argument(
        '--off',
        metavar='device_id',
        help='Switch off a given device_id',
        action='append',
    )

    parser.add_argument(
        '--lock',
        metavar='device_id',
        help='Lock a given device_id',
        action='append',
    )

    parser.add_argument(
        '--unlock',
        metavar='device_id',
        help='Unlock a given device_id',
        action='append',
    )

    parser.add_argument(
        '--automations',
        help='Output all automations',
        default=False,
        action="store_true",
    )

    parser.add_argument(
        '--activate',
        metavar='automation_id',
        help='Activate (enable) an automation by automation_id',
        action='append',
    )

    parser.add_argument(
        '--deactivate',
        metavar='automation_id',
        help='Deactivate (disable) an automation by automation_id',
        action='append',
    )

    parser.add_argument(
        '--trigger',
        metavar='automation_id',
        help='Trigger (apply) a manual (quick) automation by automation_id',
        action='append',
    )

    parser.add_argument(
        '--capture',
        metavar='device_id',
        help='Trigger a new image capture for the given device_id',
        action='append',
    )

    parser.add_argument(
        '--image',
        metavar='device_id=location/image.jpg',
        help='Save an image from a camera (if available) to the given path',
        action='append',
    )

    parser.add_argument(
        '--listen',
        help='Block and listen for device_id',
        default=False,
        action="store_true",
    )

    parser.add_argument(
        '--debug',
        help='Enable debug logging',
        default=False,
        action="store_true",
    )

    parser.add_argument(
        '--quiet',
        help='Output only warnings and errors',
        default=False,
        action="store_true",
    )

    return parser


def _create_abode_instance(args):
    return Abode(
        username=args.username,
        password=args.password,
        get_devices=args.mfa is None,
        cache_path=args.cache,
    )


@contextlib.contextmanager
def _log_errors_and_logout(abode):
    try:
        yield abode
    except abode.AbodeException as exc:
        _LOGGER.error(exc)
    finally:
        abode.logout()


def _device_print(dev, append=''):
    _LOGGER.info("%s%s", dev.desc, append)


def _device_callback(dev):
    _device_print(dev, ", At: " + time.strftime("%Y-%m-%d %H:%M:%S"))


def _timeline_callback(tl_json):
    event_code = int(tl_json['event_code'])
    if 5100 <= event_code <= 5199:
        # Ignore device changes
        return

    _LOGGER.info(
        "%s - %s at %s %s",
        tl_json['event_name'],
        tl_json['event_type'],
        tl_json['date'],
        tl_json['time'],
    )


class Dispatcher:
    def __init__(self, abode, args):
        self.abode = abode
        self.args = args

    def dispatch(self):
        self.login()
        self.output_current_mode()
        self.change_system_mode()
        self.set_setting()
        self.switch_on()
        self.switch_off()
        self.lock()
        self.unlock()
        self.output_json()
        self.print_all_automations()
        self.enable_automation()
        self.disable_automation()
        self.trigger_automation()
        self.trigger_image_capture()
        self.save_camera_image()
        self.print_all_devices()
        self.print_specific_devices()
        self.start_device_change_listener()

    def login(self):
        if not self.args.mfa:
            return
        self.abode.login(mfa_code=self.args.mfa)
        # fetch devices from Abode
        self.abode.get_devices()

    def output_current_mode(self):
        if not self.args.mode:
            return
        _LOGGER.info("Current alarm mode: %s", self.abode.get_alarm().mode)

    def change_system_mode(self):
        if not self.args.arm:
            return
        if self.abode.get_alarm().set_mode(self.args.arm):
            _LOGGER.info("Alarm mode changed to: %s", self.args.arm)
        else:
            _LOGGER.warning("Failed to change alarm mode to: %s", self.args.arm)

    def set_setting(self):
        for setting in self.args.set or []:
            keyval = setting.split("=")
            if self.abode.set_setting(keyval[0], keyval[1]):
                _LOGGER.info("Setting %s changed to %s", keyval[0], keyval[1])

    def switch_on(self):
        for device_id in self.args.on or []:
            device = self.abode.get_device(device_id)

            if device:
                if device.switch_on():
                    _LOGGER.info("Switched on device with id: %s", device_id)
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def switch_off(self):
        for device_id in self.args.off or []:
            device = self.abode.get_device(device_id)

            if device:
                if device.switch_off():
                    _LOGGER.info("Switched off device with id: %s", device_id)
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def lock(self):
        for device_id in self.args.lock or []:
            device = self.abode.get_device(device_id)

            if device:
                if device.lock():
                    _LOGGER.info("Locked device with id: %s", device_id)
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def unlock(self):
        for device_id in self.args.unlock or []:
            device = self.abode.get_device(device_id)

            if device:
                if device.unlock():
                    _LOGGER.info("Unlocked device with id: %s", device_id)
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def output_json(self):
        for device_id in self.args.json or []:
            device = self.abode.get_device(device_id)

            if device:
                # pylint: disable=protected-access
                print(
                    json.dumps(
                        device._json_state,
                        sort_keys=True,
                        indent=4,
                        separators=(',', ': '),
                    )
                )
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def print_all_automations(self):
        if not self.args.automations:
            return
        for automation in self.abode.get_automations():
            _device_print(automation)

    def enable_automation(self):
        for automation_id in self.args.activate or []:
            automation = self.abode.get_automation(automation_id)

            if automation:
                if automation.enable(True):
                    _LOGGER.info("Activated automation with id: %s", automation_id)
            else:
                _LOGGER.warning("Could not find automation with id: %s", automation_id)

    def disable_automation(self):
        for automation_id in self.args.deactivate or []:
            automation = self.abode.get_automation(automation_id)

            if automation:
                if automation.enable(False):
                    _LOGGER.info("Deactivated automation with id: %s", automation_id)
            else:
                _LOGGER.warning("Could not find automation with id: %s", automation_id)

    def trigger_automation(self):
        for automation_id in self.args.trigger or []:
            automation = self.abode.get_automation(automation_id)

            if automation:
                if automation.trigger():
                    _LOGGER.info("Triggered automation with id: %s", automation_id)
            else:
                _LOGGER.warning("Could not find automation with id: %s", automation_id)

    def trigger_image_capture(self):
        for device_id in self.args.capture or []:
            device = self.abode.get_device(device_id)

            if device:
                if device.capture():
                    _LOGGER.info("Image requested from device with id: %s", device_id)
                else:
                    _LOGGER.warning(
                        "Failed to request image from device with id: %s", device_id
                    )
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def save_camera_image(self):
        for keyval in self.args.image or []:
            devloc = keyval.split("=")
            device = self.abode.get_device(devloc[0])

            if device:
                try:
                    if device.refresh_image() and device.image_to_file(devloc[1]):
                        _LOGGER.info(
                            "Saved image to %s for device id: %s", devloc[1], devloc[0]
                        )
                except AbodeException as exc:
                    _LOGGER.warning("Unable to save image: %s", exc)
            else:
                _LOGGER.warning("Could not find device with id: %s", devloc[0])

    def print_all_devices(self):
        if not self.args.devices:
            return
        for device in self.abode.get_devices():
            _device_print(device)

    def print_specific_devices(self):
        if not self.args.device:
            return
        for device_id in self.args.device:
            device = self.abode.get_device(device_id)

            if device:
                _device_print(device)

                # Register the specific devices if we decide to listen.
                self.abode.events.add_device_callback(device_id, _device_callback)
            else:
                _LOGGER.warning("Could not find device with id: %s", device_id)

    def start_device_change_listener(self):
        if not self.args.listen:
            return
        # If no devices were specified then listen to all devices.
        if self.args.device is None:
            _LOGGER.info("Adding all devices to listener...")

            for device in self.abode.get_devices():
                self.abode.events.add_device_callback(
                    device.device_id, _device_callback
                )

        self.abode.events.add_timeline_callback(TIMELINE.ALL, _timeline_callback)

        _LOGGER.info("Listening for device and timeline updates...")
        self.abode.events.start()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.abode.events.stop()
            _LOGGER.info("Device update listening stopped.")


def main():
    """Execute command line helper."""
    args = build_parser().parse_args()

    setup_logging(log_level=logging.INFO + 10 * (args.quiet - args.debug))

    with _log_errors_and_logout(_create_abode_instance(args)) as abode:
        Dispatcher(abode, args).dispatch()


if __name__ == '__main__':
    main()



"""Mock Abode LM (Light/Temp/Humidity) Device."""
DEVICE_ID = 'ZB:eee888801'

TEMP_F = '72 °F'
TEMP_C = '72 °C'

LUX = '0 lx'

HUMIDITY = '42 %'


def device(
    devid=DEVICE_ID,
    status=TEMP_F,
    temp=TEMP_F,
    lux=LUX,
    humidity=HUMIDITY,
    low_battery=False,
    no_response=False,
):
    """PIR mock device."""
    return (
        '''
        {
          "id":"'''
        + devid
        + '''",
          "type_tag":"device_type.lm",
          "type":"LM",
          "name":"Bedroom Temp",
          "area":"1",
          "zone":"15",
          "sort_order":null,
          "is_window":"",
          "bypass":"0",
          "schar_24hr":"0",
          "sresp_mode_0":"0",
          "sresp_entry_0":"0",
          "sresp_exit_0":"0",
          "sresp_mode_1":"5",
          "sresp_entry_1":"4",
          "sresp_exit_1":"0",
          "sresp_mode_2":"0",
          "sresp_entry_2":"4",
          "sresp_exit_2":"0",
          "sresp_mode_3":"0",
          "sresp_entry_3":"0",
          "sresp_exit_3":"0",
          "version":"LMHT_00.00.03.03TC",
          "origin":"abode",
          "deep_link":null,
          "status_color":"#5cb85c",
          "faults":{
             "low_battery":'''
        + str(int(low_battery))
        + ''',
             "tempered":0,
             "supervision":0,
             "out_of_order":0,
             "no_response":'''
        + str(int(no_response))
        + '''
          },
          "status":"'''
        + status
        + '''",
          "statusEx": "0",
          "statuses":{
             "hvac_mode":null,
             "temperature": "'''
        + temp
        + '''",
             "lux": "'''
        + lux
        + '''",
             "humidity": "'''
        + humidity
        + '''"
          },
          "status_ex":"",
          "actions":[],
          "status_icons":[
          ],
          "icon":"assets/icons/unknown.svg"
       }'''
    )



"""Test the Abode device classes."""
import json
import unittest

import requests_mock

import jaraco.abode

from jaraco.abode.devices import AbodeDevice
from jaraco.abode.devices.alarm import AbodeAlarm
from jaraco.abode.devices.binary_sensor import AbodeBinarySensor
from jaraco.abode.devices.cover import AbodeCover
from jaraco.abode.devices.light import AbodeLight
from jaraco.abode.devices.lock import AbodeLock
from jaraco.abode.devices.switch import AbodeSwitch
import jaraco.abode.helpers.constants as CONST
import tests.mock.devices as DEVICES
import tests.mock.devices.door_contact as DOOR_CONTACT
import tests.mock.devices.door_lock as DOOR_LOCK
import tests.mock.devices.glass as GLASS
import tests.mock.devices.ir_camera as IR_CAMERA
import tests.mock.devices.keypad as KEYPAD
import tests.mock.devices.pir as PIR
import tests.mock.devices.power_switch_meter as POWERMETER
import tests.mock.devices.power_switch_sensor as POWERSENSOR
import tests.mock.devices.remote_controller as REMOTE_CONTROLLER
import tests.mock.devices.secure_barrier as SECUREBARRIER
import tests.mock.devices.siren as SIREN
import tests.mock.devices.status_display as STATUS_DISPLAY
import tests.mock.devices.water_sensor as WATER_SENSOR
import tests.mock.devices.unknown as UNKNOWN
import tests.mock.login as LOGIN
import tests.mock.oauth_claims as OAUTH_CLAIMS
import tests.mock.logout as LOGOUT
import tests.mock.panel as PANEL
import pytest


USERNAME = 'foobar'
PASSWORD = 'deadbeef'


class TestDevice(unittest.TestCase):
    """Test the generic AbodePy device class."""

    def setUp(self):
        """Set up Abode module."""
        self.abode = jaraco.abode.Abode(
            username=USERNAME, password=PASSWORD, disable_cache=True
        )

    def tearDown(self):
        """Clean up after test."""
        self.abode = None

    def tests_device_mapping_typetag(self):
        """Check the generic Abode device maps to none without typetag."""
        # Set up device
        device_text = GLASS.device(
            status=CONST.STATUS_ONLINE,
            low_battery=True,
            no_response=True,
            tampered=True,
            out_of_order=True,
        )

        device_json = json.loads(device_text)

        with pytest.raises(jaraco.abode.AbodeException):
            device_json['type_tag'] = ""
            jaraco.abode.new_device(device_json, self.abode)

        with pytest.raises(jaraco.abode.AbodeException):
            device_json['type_tag'] = None
            jaraco.abode.new_device(device_json, self.abode)

        with pytest.raises(jaraco.abode.AbodeException):
            del device_json['type_tag']
            jaraco.abode.new_device(device_json, self.abode)

    def tests_device_auto_naming(self):
        """Check the generic Abode device creates a name."""
        # Set up device
        device_text = GLASS.device(
            status=CONST.STATUS_ONLINE,
            low_battery=True,
            no_response=True,
            tampered=True,
            out_of_order=True,
        )

        device_json = json.loads(device_text)

        device_json['name'] = ""
        device = jaraco.abode.new_device(device_json, self.abode)
        generated_name = device.type + ' ' + device.device_id
        assert device.name == generated_name

        device_json['name'] = None
        device = jaraco.abode.new_device(device_json, self.abode)
        generated_name = device.type + ' ' + device.device_id
        assert device.name == generated_name

        del device_json['name']
        device = jaraco.abode.new_device(device_json, self.abode)
        generated_name = device.type + ' ' + device.device_id
        assert device.name == generated_name

    @requests_mock.mock()
    def tests_device_init(self, m):
        """Check the generic Abode device class init's properly."""
        # Set up URLs
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Set up device
        device_text = (
            '['
            + GLASS.device(
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
                tampered=True,
                out_of_order=True,
                uuid='testuuid00000001',
            )
            + ']'
        )
        device_json = json.loads(device_text)

        m.get(CONST.DEVICES_URL, text=device_text)

        # Logout to reset everything
        self.abode.logout()

        # Get our specific device
        device = self.abode.get_device(GLASS.DEVICE_ID)

        # Check device states match
        assert device is not None
        # pylint: disable=W0212
        assert device.name == device_json[0]['name']
        assert device.type == device_json[0]['type']
        assert device.type_tag == device_json[0]['type_tag']
        assert device.device_id == device_json[0]['id']
        assert device.device_uuid == device_json[0]['uuid']
        assert device.status == CONST.STATUS_ONLINE
        assert device.battery_low
        assert device.no_response
        assert device.tampered
        assert device.out_of_order
        assert device.desc is not None

    @requests_mock.mock()
    def tests_generic_device_refresh(self, m):
        """Check the generic Abode device class init's properly."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        # Set up online device
        device_text_online = '[' + GLASS.device(status=CONST.STATUS_ONLINE) + ']'
        m.get(CONST.DEVICES_URL, text=device_text_online)

        # Set up offline device
        device_text_offline = '[' + GLASS.device(status=CONST.STATUS_OFFLINE) + ']'
        device_url = str.replace(CONST.DEVICE_URL, '$DEVID$', GLASS.DEVICE_ID)
        m.get(device_url, text=device_text_offline)

        # Logout to reset everything
        self.abode.logout()

        # Get the first device and test
        device = self.abode.get_device(GLASS.DEVICE_ID)
        assert device.status == CONST.STATUS_ONLINE

        # Refresh the device and test
        device = self.abode.get_device(GLASS.DEVICE_ID, refresh=True)
        assert device.status == CONST.STATUS_OFFLINE

    @requests_mock.mock()
    def tests_multiple_devices(self, m):
        """Tests that multiple devices are returned properly."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))

        # Set up a list of devices
        dev_list = (
            '['
            + POWERSENSOR.device()
            + ","
            + DOOR_CONTACT.device()
            + ","
            + GLASS.device()
            + ']'
        )

        m.get(CONST.DEVICES_URL, text=dev_list)

        # Logout to reset everything
        self.abode.logout()

        # Get our devices
        devices = self.abode.get_devices()

        # Assert four devices - three from above + 1 alarm
        assert devices is not None
        assert len(devices) == 4

        # Get each individual device by device ID
        psd = self.abode.get_device(POWERSENSOR.DEVICE_ID)
        assert psd is not None

        # Get each individual device by device ID
        psd = self.abode.get_device(DOOR_CONTACT.DEVICE_ID)
        assert psd is not None

        # Get each individual device by device ID
        psd = self.abode.get_device(GLASS.DEVICE_ID)
        assert psd is not None

    @requests_mock.mock()
    def tests_unknown_devices(self, m):
        """Tests that multiple devices are returned properly."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))

        # Set up a list of devices
        dev_list = '[' + UNKNOWN.device() + ']'

        m.get(CONST.DEVICES_URL, text=dev_list)

        # Logout to reset everything
        self.abode.logout()

        # Get our devices
        devices = self.abode.get_devices()

        # Assert 1 device - skipped device above + 1 alarm
        assert devices is not None
        assert len(devices) == 1

    @requests_mock.mock()
    def tests_device_category_filter(self, m):
        """Tests that device category filter returns requested results."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))

        # Set up a list of devices
        dev_list = (
            '['
            + POWERSENSOR.device(
                devid='ps1',
                status=CONST.STATUS_OFF,
                low_battery=False,
                no_response=False,
            )
            + ","
            + POWERSENSOR.device(
                devid='ps2',
                status=CONST.STATUS_OFF,
                low_battery=False,
                no_response=False,
            )
            + ","
            + GLASS.device(
                devid='gb1',
                status=CONST.STATUS_OFF,
                low_battery=False,
                no_response=False,
            )
            + ']'
        )

        m.get(CONST.DEVICES_URL, text=dev_list)

        # Logout to reset everything
        self.abode.logout()

        # Get our glass devices
        devices = self.abode.get_devices(generic_type=(CONST.TYPE_CONNECTIVITY))

        assert devices is not None
        assert len(devices) == 1

        # Get our power switch devices
        devices = self.abode.get_devices(generic_type=(CONST.TYPE_SWITCH))

        assert devices is not None
        assert len(devices) == 2

    @requests_mock.mock()
    def tests_no_control_url(self, m):
        """Check that devices return false without control url's."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok())

        m.get(CONST.DEVICES_URL, text=GLASS.device(status=CONST.STATUS_ONLINE))

        # Logout to reset everything
        self.abode.logout()

        # Get device
        device = self.abode.get_device(GLASS.DEVICE_ID)

        assert device is not None
        assert not device.set_status('1')
        assert not device.set_level('99')

    @requests_mock.mock()
    def tests_device_status_changes(self, m):
        """Tests that device status changes work as expected."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(
            CONST.DEVICES_URL,
            text=POWERSENSOR.device(
                devid=POWERSENSOR.DEVICE_ID,
                status=CONST.STATUS_OFF,
                low_battery=False,
                no_response=False,
            ),
        )

        # Logout to reset everything
        self.abode.logout()

        # Get our power switch
        device = self.abode.get_device(POWERSENSOR.DEVICE_ID)

        # Test that we have our device
        assert device is not None
        assert device.status == CONST.STATUS_OFF
        assert not device.is_on

        # Set up control url response
        control_url = CONST.BASE_URL + POWERSENSOR.CONTROL_URL
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid=POWERSENSOR.DEVICE_ID, status=CONST.STATUS_ON_INT
            ),
        )

        # Change the mode to "on"
        assert device.switch_on()
        assert device.status == CONST.STATUS_ON
        assert device.is_on

        # Change response
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid=POWERSENSOR.DEVICE_ID, status=CONST.STATUS_OFF_INT
            ),
        )

        # Change the mode to "off"
        assert device.switch_off()
        assert device.status == CONST.STATUS_OFF
        assert not device.is_on

        # Test that an invalid device ID in response throws exception
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid='ZW:deadbeef', status=CONST.STATUS_OFF_INT
            ),
        )

        with pytest.raises(jaraco.abode.AbodeException):
            device.switch_on()

        # Test that an invalid status in response throws exception
        m.put(
            control_url,
            text=DEVICES.status_put_response_ok(
                devid=POWERSENSOR.DEVICE_ID, status=CONST.STATUS_OFF_INT
            ),
        )

        with pytest.raises(jaraco.abode.AbodeException):
            device.switch_on()

    @requests_mock.mock()
    def tests_device_level_changes(self, m):
        """Tests that device level changes work as expected."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))

        # TODO: Test with a device that supports levels
        m.get(
            CONST.DEVICES_URL,
            text=POWERSENSOR.device(
                devid=POWERSENSOR.DEVICE_ID,
                status=CONST.STATUS_OFF,
                low_battery=False,
                no_response=False,
            ),
        )

        # Logout to reset everything
        self.abode.logout()

        # Get our power switch
        device = self.abode.get_device(POWERSENSOR.DEVICE_ID)

        # Test that we have our device
        assert device is not None
        assert device.status == CONST.STATUS_OFF
        assert not device.is_on

        # Set up control url response
        control_url = CONST.BASE_URL + POWERSENSOR.CONTROL_URL
        m.put(
            control_url,
            text=DEVICES.level_put_response_ok(
                devid=POWERSENSOR.DEVICE_ID, level='100'
            ),
        )

        # Change the level to int 100
        assert device.set_level(100)
        # self.assertEqual(device.level, '100')

        # Change response
        control_url = CONST.BASE_URL + POWERSENSOR.CONTROL_URL
        m.put(
            control_url,
            text=DEVICES.level_put_response_ok(devid=POWERSENSOR.DEVICE_ID, level='25'),
        )

        # Change the level to str '25'
        assert device.set_level('25')
        # self.assertEqual(device.level, '25')

        # Test that an invalid device ID in response throws exception
        m.put(
            control_url,
            text=DEVICES.level_put_response_ok(devid='ZW:deadbeef', level='25'),
        )

        with pytest.raises(jaraco.abode.AbodeException):
            device.set_level(25)

        # Test that an invalid level in response throws exception
        m.put(
            control_url,
            text=DEVICES.level_put_response_ok(devid=POWERSENSOR.DEVICE_ID, level='98'),
        )

        with pytest.raises(jaraco.abode.AbodeException):
            device.set_level('28')

    @requests_mock.mock()
    def tests_all_devices(self, m):
        """Tests that all mocked devices are mapped correctly."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))

        # Create all devices
        all_devices = (
            '['
            + DOOR_CONTACT.device()
            + ','
            + DOOR_LOCK.device()
            + ','
            + GLASS.device()
            + ','
            + IR_CAMERA.device()
            + ','
            + KEYPAD.device()
            + ','
            + PIR.device()
            + ','
            + POWERMETER.device()
            + ','
            + POWERSENSOR.device()
            + ','
            + REMOTE_CONTROLLER.device()
            + ','
            + SECUREBARRIER.device()
            + ','
            + SIREN.device()
            + ','
            + STATUS_DISPLAY.device()
            + ','
            + WATER_SENSOR.device()
            + ']'
        )

        m.get(CONST.DEVICES_URL, text=all_devices)

        # Logout to reset everything
        self.abode.logout()

        # Loop through all devices
        for device in self.abode.get_devices():
            class_type = {
                # Alarm
                CONST.TYPE_ALARM: AbodeAlarm,
                # Binary Sensors
                CONST.TYPE_CONNECTIVITY: AbodeBinarySensor,
                CONST.TYPE_MOISTURE: AbodeBinarySensor,
                CONST.TYPE_OPENING: AbodeBinarySensor,
                CONST.TYPE_MOTION: AbodeBinarySensor,
                CONST.TYPE_OCCUPANCY: AbodeBinarySensor,
                # Camera
                CONST.TYPE_CAMERA: AbodeDevice,
                # Cover
                CONST.TYPE_COVER: AbodeCover,
                # Dimmer
                CONST.TYPE_LIGHT: AbodeLight,
                # Lock
                CONST.TYPE_LOCK: AbodeLock,
                # Switch
                CONST.TYPE_SWITCH: AbodeSwitch,
            }.get(device.generic_type)

            assert class_type is not None, device.type + ' is not mapped.'
            assert isinstance(device, class_type), (
                device.type
                + ' is of class '
                + str(device.__class__.__name__)
                + ' but mapped to '
                + str(class_type.__name__)
            )



"""Mock Abode Claims Response."""

from tests.mock import OAUTH_TOKEN


def get_response_ok(oauth_token=OAUTH_TOKEN):
    """Return the oauth2 claims token."""
    return (
        '''
    {
        "token_type":"Bearer",
        "access_token":"'''
        + oauth_token
        + '''",
        "expires_in":3600
    }'''
    )



"""Mock Abode IR Camera Device."""
import jaraco.abode.helpers.constants as CONST

DEVICE_ID = 'ZB:00000005'
CONTROL_URL = 'api/v1/cams/' + DEVICE_ID + '/capture'


def device(
    devid=DEVICE_ID, status=CONST.STATUS_ONLINE, low_battery=False, no_response=False
):
    """IR camera mock device."""
    return (
        '''
        {
          "id":"'''
        + devid
        + '''",
          "type_tag":"device_type.ir_camera",
          "type":"Motion Camera",
          "name":"Downstairs Motion Camera",
          "area":"1",
          "zone":"3",
          "sort_order":null,
          "is_window":"",
          "bypass":"0",
          "schar_24hr":"0",
          "sresp_mode_0":"0",
          "sresp_entry_0":"0",
          "sresp_exit_0":"0",
          "sresp_mode_1":"5",
          "sresp_entry_1":"4",
          "sresp_exit_1":"0",
          "sresp_mode_2":"0",
          "sresp_entry_2":"4",
          "sresp_exit_2":"0",
          "sresp_mode_3":"0",
          "sresp_entry_3":"0",
          "sresp_exit_3":"0",
          "version":"852_00.00.03.05TC",
          "origin":"abode",
          "control_url":"'''
        + CONTROL_URL
        + '''",
          "deep_link":null,
          "status_color":"#5cb85c",
          "faults":{
             "low_battery":'''
        + str(int(low_battery))
        + ''',
             "tempered":0,
             "supervision":0,
             "out_of_order":0,
             "no_response":'''
        + str(int(no_response))
        + '''
          },
          "status":"'''
        + status
        + '''",
          "statuses":{
             "hvac_mode":null
          },
          "status_ex":"",
          "actions":[
             {
                "label":"Auto Flash",
                "value":"a=1&z=3&req=img;"
             },
             {
                "label":"Never Flash",
                "value":"a=1&z=3&req=img_nf;"
             }
          ],
          "status_icons":[
          ],
          "motion_event":"1",
          "wide_angle":"0",
          "icon":"assets/icons/motioncamera-b.svg"
       }'''
    )


def get_capture_timeout():
    """Mock timeout response."""
    return '''
    {
        "code":600,
        "message":"Image Capture request has timed out.",
        "title":"",
        "detail":null
    }'''


FILE_PATH_ID = 'ZB00000005'
FILE_PATH = 'api/storage/' + FILE_PATH_ID + '/2017-08-23/195505UTC/001.jpg'

LOCATION_HEADER = (
    'https://www.google.com/images/branding/googlelogo/'
    + '1x/googlelogo_color_272x92dp.png'
)


def timeline_event(devid=DEVICE_ID, event_code='5001', file_path=FILE_PATH):
    """Camera Timeline Event Mockup."""
    return (
        '''
    {
        "id": "71739948",
        "event_utc": "1503518105",
        "nest_activity_zones": null,
        "nest_has_motion": null,
        "nest_has_sound": null,
        "nest_has_person": null,
        "date": "08/23/2017",
        "time": "12:55 PM",
        "is_alarm": "0",
        "event_cid": "",
        "event_code": "'''
        + event_code
        + '''",
        "device_id": "'''
        + devid
        + '''",
        "device_type_id": "27",
        "device_type": "Motion Camera",
        "device_name": "Downstairs Motion Camera",
        "file_path":"'''
        + file_path
        + '''",
        "deep_link": null,
        "app_deep_link": null,
        "file_size": "30852",
        "file_count": "1",
        "file_is_del": "0",
        "event_type": "Image Capture",
        "severity": "6",
        "pos": "l",
        "color": "#40bbea",
        "viewed_by_uid": "",
        "user_id": "1234",
        "user_name": "Wil",
        "mobile_name": "",
        "parent_tid": "",
        "icon": "assets/email/motion-camera.png",
        "app_type": "WebApp",
        "file_del_at": "",
        "event_name": "Downstairs Motion Camera Image Capture",
        "event_by": ""
    }'''
    )



"""Test the Abode camera class."""
import os
import unittest

import requests_mock

import jaraco.abode
from jaraco.abode.exceptions import AbodeException
import jaraco.abode.helpers.constants as CONST
import jaraco.abode.helpers.errors as ERROR
import tests.mock as MOCK
import tests.mock.devices.ipcam as IPCAM
import tests.mock.devices.ir_camera as IRCAMERA
import tests.mock.login as LOGIN
import tests.mock.logout as LOGOUT
import tests.mock.oauth_claims as OAUTH_CLAIMS
import tests.mock.panel as PANEL
import pytest

USERNAME = "foobar"
PASSWORD = "deadbeef"


def set_cam_type(device_type):
    """Return camera type_tag."""
    if device_type == CONST.DEVICE_IP_CAM:
        return IPCAM

    if device_type == CONST.DEVICE_MOTION_CAMERA:
        return IRCAMERA


@requests_mock.Mocker()
class TestCamera(unittest.TestCase):
    """Test the AbodePy camera."""

    def setUp(self):
        """Set up Abode module."""
        self.abode = jaraco.abode.Abode(
            username=USERNAME, password=PASSWORD, disable_cache=True
        )

        self.all_devices = (
            "["
            + IRCAMERA.device(
                devid=IRCAMERA.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=False,
                no_response=False,
            )
            + ","
            + IPCAM.device(
                devid=IPCAM.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=False,
                no_response=False,
            )
            + "]"
        )

        # Logout to reset everything
        self.abode.logout()

    def tearDown(self):
        """Clean up after test."""
        self.abode = None

    def tests_camera_properties(self, m):
        """Tests that camera properties work as expected."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(CONST.DEVICES_URL, text=self.all_devices)

        # Get our camera
        for device in self.abode.get_devices():
            # Skip alarm devices
            if device.type_tag == CONST.DEVICE_ALARM:
                continue

            # Specify which device module to use based on type_tag
            cam_type = set_cam_type(device.type_tag)

            # Test our device
            assert device is not None
            assert device.status == CONST.STATUS_ONLINE
            assert not device.battery_low
            assert not device.no_response

            # Set up our direct device get url
            device_url = str.replace(CONST.DEVICE_URL, "$DEVID$", device.device_id)

            # Change device properties
            m.get(
                device_url,
                text=cam_type.device(
                    devid=cam_type.DEVICE_ID,
                    status=CONST.STATUS_OFFLINE,
                    low_battery=True,
                    no_response=True,
                ),
            )

            # Refesh device and test changes
            device.refresh()

            assert device.status == CONST.STATUS_OFFLINE
            assert device.battery_low
            assert device.no_response

    def tests_camera_capture(self, m):
        """Tests that camera devices capture new images."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(CONST.DEVICES_URL, text=self.all_devices)

        # Test our camera devices
        for device in self.abode.get_devices():
            # Skip alarm devices
            if device.type_tag == CONST.DEVICE_ALARM:
                continue

            # Specify which device module to use based on type_tag
            cam_type = set_cam_type(device.type_tag)

            # Test that we have the camera devices
            assert device is not None
            assert device.status == CONST.STATUS_ONLINE

            # Determine URL based on device type
            if device.type_tag == CONST.DEVICE_IP_CAM:
                url = CONST.BASE_URL + cam_type.CONTROL_URL_SNAPSHOT

            elif device.type_tag == CONST.DEVICE_MOTION_CAMERA:
                url = CONST.BASE_URL + cam_type.CONTROL_URL

            # Set up capture URL response
            m.put(url, text=MOCK.generic_response_ok())

            # Capture an image
            assert device.capture()

            # Change capture URL responses
            m.put(url, text=cam_type.get_capture_timeout(), status_code=600)

            # Capture an image with a failure
            assert not device.capture()

            # Remove control URLs from JSON to test if Abode makes
            # changes to JSON
            # pylint: disable=protected-access
            for key in list(device._json_state.keys()):
                if key.startswith("control_url"):
                    # pylint: disable=protected-access
                    del device._json_state[key]

            # Test that AbodeException is raised with no control URLs
            with pytest.raises(AbodeException) as exc:
                device.capture()
                assert str(exc.exception) == ERROR.MISSING_CONTROL_URL

    def tests_camera_image_update(self, m):
        """Tests that camera devices update correctly via timeline request."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(CONST.DEVICES_URL, text=self.all_devices)

        # Test our camera devices
        for device in self.abode.get_devices():
            # Skip alarm devices
            if device.type_tag == CONST.DEVICE_ALARM:
                continue

            # Specify which device module to use based on type_tag
            cam_type = set_cam_type(device.type_tag)

            # Test that we have our device
            assert device is not None
            assert device.status == CONST.STATUS_ONLINE

            # Set up timeline response
            url = str.replace(CONST.TIMELINE_IMAGES_ID_URL, "$DEVID$", device.device_id)

            m.get(url, text="[" + cam_type.timeline_event(device.device_id) + "]")
            # Set up our file path response
            file_path = CONST.BASE_URL + cam_type.FILE_PATH
            m.head(
                file_path,
                status_code=302,
                headers={"Location": cam_type.LOCATION_HEADER},
            )

            # Refresh the image
            assert device.refresh_image()

            # Verify the image location
            assert device.image_url == cam_type.LOCATION_HEADER

            # Test that a bad file_path response header results in an exception
            file_path = CONST.BASE_URL + cam_type.FILE_PATH
            m.head(file_path, status_code=302)

            with pytest.raises(jaraco.abode.AbodeException):
                device.refresh_image()

            # Test that a bad file_path response code results in an exception
            file_path = CONST.BASE_URL + cam_type.FILE_PATH
            m.head(
                file_path,
                status_code=200,
                headers={"Location": cam_type.LOCATION_HEADER},
            )

            with pytest.raises(jaraco.abode.AbodeException):
                device.refresh_image()

            # Test that an an empty timeline event throws exception
            url = str.replace(CONST.TIMELINE_IMAGES_ID_URL, "$DEVID$", device.device_id)
            m.get(
                url,
                text="["
                + cam_type.timeline_event(device.device_id, file_path="")
                + "]",
            )

            with pytest.raises(jaraco.abode.AbodeException):
                device.refresh_image()

            # Test that an unexpected timeline event throws exception
            url = str.replace(CONST.TIMELINE_IMAGES_ID_URL, "$DEVID$", device.device_id)
            m.get(
                url,
                text="["
                + cam_type.timeline_event(device.device_id, event_code="1234")
                + "]",
            )

            with pytest.raises(jaraco.abode.AbodeException):
                device.refresh_image()

    def tests_camera_no_image_update(self, m):
        """Tests that camera updates correctly with no timeline events."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(CONST.DEVICES_URL, text=self.all_devices)

        # Test our camera devices
        for device in self.abode.get_devices():
            # Skip alarm devices
            if device.type_tag == CONST.DEVICE_ALARM:
                continue

            # Test that we have our device
            assert device is not None
            assert device.status == CONST.STATUS_ONLINE

            # Set up timeline response
            url = str.replace(CONST.TIMELINE_IMAGES_ID_URL, "$DEVID$", device.device_id)
            m.get(url, text="[]")

            # Refresh the image
            assert not device.refresh_image()
            assert device.image_url is None

    def tests_camera_image_write(self, m):
        """Tests that camera images will write to a file."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(CONST.DEVICES_URL, text=self.all_devices)

        # Test our camera devices
        for device in self.abode.get_devices():
            # Skip alarm devices
            if device.type_tag == CONST.DEVICE_ALARM:
                continue

            # Specify which device module to use based on type_tag
            cam_type = set_cam_type(device.type_tag)

            # Test that we have our device
            assert device is not None
            assert device.status == CONST.STATUS_ONLINE

            # Set up timeline response
            url = str.replace(CONST.TIMELINE_IMAGES_ID_URL, "$DEVID$", device.device_id)
            m.get(url, text="[" + cam_type.timeline_event(device.device_id) + "]")

            # Set up our file path response
            file_path = CONST.BASE_URL + cam_type.FILE_PATH
            m.head(
                file_path,
                status_code=302,
                headers={"Location": cam_type.LOCATION_HEADER},
            )

            # Set up our image response
            image_response = "this is a beautiful jpeg image"
            m.get(cam_type.LOCATION_HEADER, text=image_response)

            # Refresh the image
            path = "test.jpg"
            assert device.image_to_file(path, get_image=True)

            # Test the file written and cleanup
            image_data = open(path, "r").read()
            assert image_response, image_data
            os.remove(path)

            # Test that bad response returns False
            m.get(cam_type.LOCATION_HEADER, status_code=400)
            with pytest.raises(jaraco.abode.AbodeException):
                device.image_to_file(path, get_image=True)

            # Test that the image fails to update returns False
            m.get(url, text="[]")
            assert not device.image_to_file(path, get_image=True)

    def tests_camera_privacy_mode(self, m):
        """Tests camera privacy mode."""
        # Set up mock URLs
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))
        m.get(CONST.DEVICES_URL, text=self.all_devices)

        # Get the IP camera and test we have it
        device = self.abode.get_device(IPCAM.DEVICE_ID)
        assert device is not None
        assert device.status == CONST.STATUS_ONLINE

        # Set up params URL response for privacy mode on
        m.put(CONST.PARAMS_URL + device.device_id, text=IPCAM.device(privacy=1))

        # Set privacy mode on
        assert device.privacy_mode(True)

        # Set up params URL response for privacy mode off
        m.put(CONST.PARAMS_URL + device.device_id, text=IPCAM.device(privacy=0))

        # Set privacy mode off
        assert device.privacy_mode(False)

        # Test that an invalid privacy response throws exception
        with pytest.raises(jaraco.abode.AbodeException):
            device.privacy_mode(True)



import sys
import warnings

import jaraco.abode


for name in tuple(sys.modules):
    if not name.startswith('jaraco.abode.'):
        continue
    orig_name = name.replace('jaraco.abode.', 'abodepy.')
    sys.modules[orig_name] = sys.modules[name]

del sys
globals().update(vars(jaraco.abode))
del jaraco


warnings.warn(
    "abodepy is deprecated. Import from jaraco.abode instead.",
    stacklevel=2,
)



"""Mock Abode Automation."""


def get_response_ok(name, enabled, aid):
    """Return automation json."""
    return (
        '''{
        "name": "'''
        + name
        + '''",
        "enabled": "'''
        + str(enabled)
        + '''",
        "version": 2,
        "id": "'''
        + aid
        + '''",
        "subType": "",
        "actions": [{
            "directive": {
                "trait": "panel.traits.panelMode",
                "name": "panel.directives.arm",
                "state": {
                    "panelMode": "AWAY"
                }
            }
        }],
        "conditions": {},
        "triggers": {
            "operator": "OR",
            "expressions": [{
                "mobileDevices": ["89381", "658"],
                "property": {
                    "trait": "mobile.traits.location",
                    "name": "location",
                    "rule": {
                        "location": "31675",
                        "equalTo": "LAST_OUT"
                    }
                }
            }]
        }
    }'''
    )



"""Timeline event constants."""

# Timeline event groups.

ALARM_GROUP = 'abode_alarm'
ALARM_END_GROUP = 'abode_alarm_end'
PANEL_FAULT_GROUP = 'abode_panel_fault'
PANEL_RESTORE_GROUP = 'abode_panel_restore'
DISARM_GROUP = 'abode_disarm'
ARM_GROUP = 'abode_arm'
ARM_FAULT_GROUP = 'abode_arm_fault'
TEST_GROUP = 'abode_test'
CAPTURE_GROUP = 'abode_capture'
DEVICE_GROUP = 'abode_device'
AUTOMATION_GROUP = 'abode_automation'
AUTOMATION_EDIT_GROUP = 'abode_automation_edited'

ALL_EVENT_GROUPS = [
    ALARM_GROUP,
    ALARM_END_GROUP,
    PANEL_FAULT_GROUP,
    PANEL_RESTORE_GROUP,
    DISARM_GROUP,
    ARM_GROUP,
    ARM_FAULT_GROUP,
    TEST_GROUP,
    CAPTURE_GROUP,
    DEVICE_GROUP,
    AUTOMATION_GROUP,
    AUTOMATION_EDIT_GROUP,
]


def map_event_code(event_code):  # noqa: C901
    """Map a specific event_code to an event group."""
    event_code = int(event_code)

    # Honestly, these are just guessing based on the below event list.
    # It could be wrong, I have no idea.
    if 1100 <= event_code <= 1199:
        return ALARM_GROUP

    if 3100 <= event_code <= 3199:
        return ALARM_END_GROUP

    if 1300 <= event_code <= 1399:
        return PANEL_FAULT_GROUP

    if 3300 <= event_code <= 3399:
        return PANEL_RESTORE_GROUP

    if 1400 <= event_code <= 1499:
        return DISARM_GROUP

    if 3400 <= event_code <= 3799:
        return ARM_GROUP

    if 1600 <= event_code <= 1699:
        return TEST_GROUP

    if 5000 <= event_code <= 5099:
        return CAPTURE_GROUP

    if 5100 <= event_code <= 5199:
        return DEVICE_GROUP

    if 5200 <= event_code <= 5299:
        return AUTOMATION_GROUP

    if 6000 <= event_code <= 6100:
        return ARM_FAULT_GROUP


# Specific timeline events by event code.

ALL = {'event_code': '0', 'event_type': 'All Timeline Events (AbodePy)'}

MEDICAL = {'event_code': '1100', 'event_type': 'Medical'}

PERSONAL_EMERGENCY = {'event_code': '1101', 'event_type': 'Personal Emergency'}

FIRE_ALERT = {'event_code': '1110', 'event_type': 'Fire Alert'}

SMOKE_DETECTED = {'event_code': '1111', 'event_type': 'Smoke Detected'}

PANIC_ALERT = {'event_code': '1120', 'event_type': 'Panic Alert'}

DURESS_ALARM = {'event_code': '1121', 'event_type': 'Duress Alarm'}

SILENT_PANIC_ALERT = {'event_code': '1122', 'event_type': 'Silent Panic Alert'}

ALARM_TRIGGERED = {'event_code': '1130', 'event_type': 'Alarm Triggered'}

PERIMETER_ALARM_TRIGGERED = {
    'event_code': '1131',
    'event_type': 'Perimeter Alarm Triggered',
}

INTERIOR_ALARM_TRIGGERED = {
    'event_code': '1132',
    'event_type': 'Interior Alarm Triggered',
}

BURGLAR_ALARM_TRIGGERED = {
    'event_code': '1133',
    'event_type': 'Burglar Alarm Triggered',
}

OUTDOOR_ALARM_TRIGGERED = {
    'event_code': '1136',
    'event_type': 'Outdoor Alarm Triggered',
}

PANEL_TAMPER_SWITCH_TRIGGERED = {
    'event_code': '1137',
    'event_type': 'Panel Tamper Switch Triggered',
}

NOT_CONNECTED_TO_GATEWAY = {
    'event_code': '1147',
    'event_type': 'Not Connected to Gateway',
}

GAS_DETECTED = {'event_code': '1151', 'event_type': 'Gas Detected'}

WATER_LEAK_DETECTED = {'event_code': '1154', 'event_type': 'Water Leak Detected'}

CO_DETECTED = {'event_code': '1162', 'event_type': 'CO Detected'}

POWER_LOST = {'event_code': '1301', 'event_type': 'Power Lost'}

BATTERY_LOW = {'event_code': '1302', 'event_type': 'Battery Low'}

BATTERY_MISSING_DEAD = {'event_code': '1311', 'event_type': 'Battery Missing/Dead'}

JAM_DETECT = {'event_code': '1344', 'event_type': 'Jam Detect/Signal Interference'}

GPS_LOCATION_FAIL = {'event_code': '1354', 'event_type': 'GPS Location Fail'}

POLLING_FAILURE = {'event_code': '1355', 'event_type': 'Polling Failure'}

ARMED_WITH_FAULT = {'event_code': '1374', 'event_type': 'Armed with Fault'}

SENSOR_OFFLINE = {'event_code': '1380', 'event_type': 'Sensor Offline'}

SUPERVISION_FAIL = {'event_code': '1381', 'event_type': 'Supervision Fail'}

TAMPER_SWITCH_TRIGGERED = {
    'event_code': '1383',
    'event_type': 'Tamper Switch Triggered',
}

BATTERY_LOW = {'event_code': '1384', 'event_type': 'Battery Low'}

SYSTEM_DISARMED = {'event_code': '1400', 'event_type': 'System Disarmed'}

SYSTEM_DISARMED = {'event_code': '1401', 'event_type': 'System Disarmed'}

EVENT_CANCELED = {'event_code': '1406', 'event_type': 'Event Canceled'}

SYSTEM_DISARMED = {'event_code': '1407', 'event_type': 'System Disarmed'}

SET_UNSET_DISARM = {'event_code': '1408', 'event_type': 'Set/Unset Disarm'}

OFFLINE = {'event_code': '1447', 'event_type': 'Offline'}

FAIL_TO_CLOSE = {'event_code': '1454', 'event_type': 'Fail to Close'}

PARTIAL_ARM = {'event_code': '1456', 'event_type': 'Partial Arm'}

USER_ON_PREMISES = {'event_code': '1458', 'event_type': 'User on Premises'}

RECENT_CLOSE = {'event_code': '1459', 'event_type': 'Recent Close'}

ZONE_BYPASSED = {'event_code': '1570', 'event_type': 'Zone Bypassed'}

MANUAL_TEST_REPORT = {'event_code': '1601', 'event_type': 'Manual Test Report'}

PERIODIC_TEST = {'event_code': '1602', 'event_type': 'Periodic Test'}

POINT_TESTED_OK = {
    'event_code': '1611',
    'event_type': 'Point Tested OK/Technical Alarm',
}

CALL_REQUEST = {'event_code': '1616', 'event_type': 'Call Request'}

MOBILITY_ALARM = {
    'event_code': '1641',
    'event_type': 'Mobility Alarm/Sensor Watch Trouble',
}

GPS_LOCATION_HELP = {'event_code': '1645', 'event_type': 'GPS Location Help'}

GPS_LOCATION_REQUEST = {'event_code': '1646', 'event_type': 'GPS Location Request'}

GPS_LOCATION_TRACKER = {'event_code': '1647', 'event_type': 'GPS Location Tracker'}

SCREAM = {'event_code': '1648', 'event_type': 'Scream'}

MOBILE_UNIT_DISCONNECTED_FROM_BASE = {
    'event_code': '1649',
    'event_type': 'Mobile Unit Disconnected from Base',
}

TEST_REPORT = {'event_code': '1655', 'event_type': 'Test Report'}

ENTRY = {'event_code': '1704', 'event_type': 'Entry'}

DC_OPEN_MOBILITY = {'event_code': '1750', 'event_type': 'DC Open - Mobility'}

IR_ACTIVITY_MOBILITY = {'event_code': '1751', 'event_type': 'IR Activity - Mobility'}

SIREN_ON = {'event_code': '1752', 'event_type': 'Siren On'}

MOTION_DETECTED_AREA_2 = {'event_code': '1788', 'event_type': 'Motion Detected Area 2'}

MOTION_DETECTED_AREA_1 = {'event_code': '1789', 'event_type': 'Motion Detected Area 1'}

PANEL_TAMPER_SWITCH_RESTORED = {
    'event_code': '3137',
    'event_type': 'Panel Tamper Switch Restored',
}

RECONNECTED_TO_GATEWAY = {'event_code': '3147', 'event_type': 'Re-Connected to Gateway'}

POWER_RESTORED = {'event_code': '3301', 'event_type': 'Power Restored'}

BATTERY_NORMAL = {'event_code': '3302', 'event_type': 'Battery Normal/OK'}

BATTERY_NORMAL = {'event_code': '3311', 'event_type': 'Battery Normal/OK'}

SIGNAL_RESTORED = {'event_code': '3344', 'event_type': 'No Jam/Signal Restored'}

NET_DEVICE_FAILURE_RESTORED = {
    'event_code': '3354',
    'event_type': 'NET Device Failure Restored',
}

POLLING_FAILURE_RESTORED = {
    'event_code': '3355',
    'event_type': 'Polling Failure Restored',
}

TAMPER_SWITCH_RESTORED = {'event_code': '3383', 'event_type': 'Tamper Switch Restored'}

BATTERY_NORMAL = {'event_code': '3384', 'event_type': 'Battery Normal/OK'}

SYSTEM_ARMED = {'event_code': '3400', 'event_type': 'System Armed'}

SYSTEM_ARMED_AWAY = {'event_code': '3401', 'event_type': 'System Armed - Away'}

SYSTEM_ARMED = {'event_code': '3407', 'event_type': 'System Armed'}

SET_UNSET_ARM = {'event_code': '3408', 'event_type': 'Set/Unset Arm'}

ONLINE = {'event_code': '3447', 'event_type': 'Online'}

SYSTEM_ARMED_HOME = {'event_code': '3456', 'event_type': 'System Armed - Home'}

MOBILE_UNIT_CONNECTED_TO_BASE = {
    'event_code': '3649',
    'event_type': 'Mobile Unit Connected to Base',
}

DC_CLOSE_MOBILITY = {'event_code': '3750', 'event_type': 'DC Close - Mobility'}

SIREN_OFF = {'event_code': '3752', 'event_type': 'Siren Off'}

SYSTEM_ARMED_HOME = {'event_code': '3758', 'event_type': 'System Armed - Home'}

VEVENT_CODEEO = {'event_code': '5000', 'event_type': 'Vevent_codeeo'}

CAPTURE_IMAGE = {'event_code': '5001', 'event_type': 'Capture Image'}

BURGLAR_VEVENT_CODEEO = {'event_code': '5002', 'event_type': 'Burglar Vevent_codeeo'}

BURGLAR_CAPTURE_IMAGE = {'event_code': '5003', 'event_type': 'Burglar Capture Image'}

OPENED = {'event_code': '5100', 'event_type': 'Opened'}

CLOSED = {'event_code': '5101', 'event_type': 'Closed'}

UNLOCKED = {'event_code': '5110', 'event_type': 'Unlocked'}

LOCKED = {'event_code': '5111', 'event_type': 'Locked'}

STATUS_AUTOMATION = {'event_code': '5201', 'event_type': 'Status Automation'}

SCHEDULED_AUTOMATION = {'event_code': '5202', 'event_type': 'Scheduled Automation'}

LOCATION_AUTOMATION = {'event_code': '5203', 'event_type': 'Location Automation'}

QUICK_ACTION = {'event_code': '5204', 'event_type': 'Quick Action'}

AUTOMATION = {'event_code': '5206', 'event_type': 'Automation'}

ARMING_WITH_FAULT_AWAY = {
    'event_code': '6055',
    'event_type': 'Exit Time Started - Arming w/ Faults - Away',
}

ARMED_WITH_FAULT_AWAY = {'event_code': '6071', 'event_type': 'Armed w/ Faults - Away'}

ARMED_WITH_FAULT_HOME = {'event_code': '6077', 'event_type': 'Armed w/ Faults - Home'}



"""Mock Abode Key Pad Device."""
import jaraco.abode.helpers.constants as CONST

DEVICE_ID = 'RF:00000002'


def device(
    devid=DEVICE_ID, status=CONST.STATUS_ONLINE, low_battery=False, no_response=False
):
    """Key pad mock device."""
    return (
        '''
       {
          "id":"'''
        + devid
        + '''",
          "type_tag":"device_type.keypad",
          "type":"Keypad",
          "name":"Keypad",
          "area":"1",
          "zone":"10",
          "sort_order":null,
          "is_window":"",
          "bypass":"0",
          "schar_24hr":"0",
          "sresp_mode_0":"5",
          "sresp_entry_0":"5",
          "sresp_exit_0":"5",
          "sresp_mode_1":"5",
          "sresp_entry_1":"5",
          "sresp_exit_1":"5",
          "sresp_mode_2":"5",
          "sresp_entry_2":"5",
          "sresp_exit_2":"5",
          "sresp_mode_3":"5",
          "sresp_entry_3":"5",
          "sresp_exit_3":"5",
          "version":"",
          "origin":"abode",
          "control_url":"",
          "deep_link":null,
          "status_color":"#5cb85c",
          "faults":{
             "low_battery":'''
        + str(int(low_battery))
        + ''',
             "tempered":0,
             "supervision":0,
             "out_of_order":0,
             "no_response":'''
        + str(int(no_response))
        + '''
          },
          "status":"'''
        + status
        + '''",
          "statuses":{
             "hvac_mode":null
          },
          "status_ex":"",
          "actions":[
          ],
          "status_icons":[
          ],
          "icon":"assets/icons/keypad-b.svg"
       }'''
    )



"""Test the Abode binary sensors."""
import unittest

import requests_mock

import jaraco.abode
import jaraco.abode.helpers.constants as CONST

import tests.mock.login as LOGIN
import tests.mock.oauth_claims as OAUTH_CLAIMS
import tests.mock.logout as LOGOUT
import tests.mock.panel as PANEL
import tests.mock.devices.door_contact as DOOR_CONTACT
import tests.mock.devices.glass as GLASS
import tests.mock.devices.keypad as KEYPAD
import tests.mock.devices.remote_controller as REMOTE_CONTROLLER
import tests.mock.devices.siren as SIREN
import tests.mock.devices.status_display as STATUS_DISPLAY
import tests.mock.devices.water_sensor as WATER_SENSOR


USERNAME = 'foobar'
PASSWORD = 'deadbeef'


class TestBinarySensors(unittest.TestCase):
    """Test the AbodePy binary sensors."""

    def setUp(self):
        """Set up Abode module."""
        self.abode = jaraco.abode.Abode(
            username=USERNAME, password=PASSWORD, disable_cache=True
        )

    def tearDown(self):
        """Clean up after test."""
        self.abode = None

    @requests_mock.mock()
    def tests_binary_sensor_properties(self, m):
        """Tests that binary sensor device properties work as expected."""
        # Set up URL's
        m.post(CONST.LOGIN_URL, text=LOGIN.post_response_ok())
        m.get(CONST.OAUTH_TOKEN_URL, text=OAUTH_CLAIMS.get_response_ok())
        m.post(CONST.LOGOUT_URL, text=LOGOUT.post_response_ok())
        m.get(CONST.PANEL_URL, text=PANEL.get_response_ok(mode=CONST.MODE_STANDBY))

        # Set up all Binary Sensor Devices in "off states"
        all_devices = (
            '['
            + DOOR_CONTACT.device(
                devid=DOOR_CONTACT.DEVICE_ID,
                status=CONST.STATUS_CLOSED,
                low_battery=False,
                no_response=False,
            )
            + ','
            + GLASS.device(
                devid=GLASS.DEVICE_ID,
                status=CONST.STATUS_OFFLINE,
                low_battery=False,
                no_response=False,
            )
            + ','
            + KEYPAD.device(
                devid=KEYPAD.DEVICE_ID,
                status=CONST.STATUS_OFFLINE,
                low_battery=False,
                no_response=False,
            )
            + ','
            + REMOTE_CONTROLLER.device(
                devid=REMOTE_CONTROLLER.DEVICE_ID,
                status=CONST.STATUS_OFFLINE,
                low_battery=False,
                no_response=False,
            )
            + ','
            + SIREN.device(
                devid=SIREN.DEVICE_ID,
                status=CONST.STATUS_OFFLINE,
                low_battery=False,
                no_response=False,
            )
            + ','
            + STATUS_DISPLAY.device(
                devid=STATUS_DISPLAY.DEVICE_ID,
                status=CONST.STATUS_OFFLINE,
                low_battery=False,
                no_response=False,
            )
            + ','
            + WATER_SENSOR.device(
                devid=WATER_SENSOR.DEVICE_ID,
                status=CONST.STATUS_OFFLINE,
                low_battery=False,
                no_response=False,
            )
            + ']'
        )

        m.get(CONST.DEVICES_URL, text=all_devices)

        # Logout to reset everything
        self.abode.logout()

        # Test our devices
        for device in self.abode.get_devices():
            # Skip alarm devices
            if device.type == CONST.DEVICE_ALARM:
                continue

            assert not device.is_on, device.type + " is_on failed"
            assert not device.battery_low, device.type + " battery_low failed"
            assert not device.no_response, device.type + " no_response failed"

        # Set up all Binary Sensor Devices in "off states"
        all_devices = (
            '['
            + DOOR_CONTACT.device(
                devid=DOOR_CONTACT.DEVICE_ID,
                status=CONST.STATUS_OPEN,
                low_battery=True,
                no_response=True,
            )
            + ','
            + GLASS.device(
                devid=GLASS.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
            )
            + ','
            + KEYPAD.device(
                devid=KEYPAD.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
            )
            + ','
            + REMOTE_CONTROLLER.device(
                devid=REMOTE_CONTROLLER.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
            )
            + ','
            + SIREN.device(
                devid=SIREN.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
            )
            + ','
            + STATUS_DISPLAY.device(
                devid=STATUS_DISPLAY.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
            )
            + ','
            + WATER_SENSOR.device(
                devid=WATER_SENSOR.DEVICE_ID,
                status=CONST.STATUS_ONLINE,
                low_battery=True,
                no_response=True,
            )
            + ']'
        )

        m.get(CONST.DEVICES_URL, text=all_devices)

        # Refesh devices and test changes
        for device in self.abode.get_devices(refresh=True):
            # Skip alarm devices
            if device.type_tag == CONST.DEVICE_ALARM:
                continue

            assert device.is_on, device.type + " is_on failed"
            assert device.battery_low, device.type + " battery_low failed"
            assert device.no_response, device.type + " no_response failed"
